from .train.regression import *
from .downloader import download_from_drive
import os
import deepspeed
import json

class SEScore2:

    def __init__(self, lang):
        # load in the weights of SEScore2
        exp_config.device_id = torch.device("cpu") #torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        if lang == 'en':
            if not os.path.exists("sescore2_en_supervised_3B.ckpt"):
                download_from_drive(FILE_ID='1XBfjvNbm5tpxdD62_gVpKQhwwrqXyewt', FILENAME='sescore2_en_supervised_3B.ckpt')
            # load in english version
            self.tokenizer = AutoTokenizer.from_pretrained(f"google/mt5-xl")
            model_named_dict = torch.load('sescore2_en_supervised_3B.ckpt', map_location=torch.device('cpu'))
            # for key in list(model_named_dict.keys()):
            #    model_named_dict[key.replace('roberta.', '')] = model_named_dict.pop(key)
            hidden_size = 2048
            exp_config.hidden_size = hidden_size
            exp_config.hidden_size_FNN = [hidden_size * 2, hidden_size]
            self.model = Regression_XLM_Roberta(f"google/mt5-xl")

            self.model.load_state_dict(model_named_dict)  # .to(exp_config.device_id)
            self.model.to(exp_config.device_id)
            self.model.eval()
        elif lang == 'de':
            # load in english version
            if not os.path.exists("sescore2_de_supervised_3B.ckpt"):
                download_from_drive(FILE_ID='1KAbZkIwfdSrYNszAFOnD1UnzOcM5J1mf', FILENAME='sescore2_de_supervised_3B.ckpt')
            self.tokenizer = AutoTokenizer.from_pretrained(f"google/mt5-xl")
            model_named_dict = torch.load('sescore2_de_supervised_3B.ckpt', map_location=torch.device('cpu'))
            # for key in list(model_named_dict.keys()):
            #    model_named_dict[key.replace('roberta.', '')] = model_named_dict.pop(key)
            hidden_size = 2048
            exp_config.hidden_size = hidden_size
            exp_config.hidden_size_FNN = [hidden_size * 2, hidden_size]
            self.model = Regression_XLM_Roberta(f"google/mt5-xl")

            self.model.load_state_dict(model_named_dict)  # .to(exp_config.device_id)
            self.model.to(exp_config.device_id)
            self.model.eval()
        elif lang == 'ja':
            # load in japanese version
            self.tokenizer = AutoTokenizer.from_pretrained(f"google/rembert")
            self.model = torch.load('sescore2_ja.ckpt').to(exp_config.device_id)
        elif lang == 'es':
            # load in spainish version
            self.tokenizer = AutoTokenizer.from_pretrained(f"google/rembert")
            self.model = torch.load('sescore2_es.ckpt').to(exp_config.device_id)
        elif lang == 'zh':
            # load in chinese version
            self.tokenizer = AutoTokenizer.from_pretrained(f"google/rembert")
            self.model = torch.load('sescore2_zh.ckpt').to(exp_config.device_id)
        else:
            print("We currently only support five languages: en, de, ja, es, zh!")
            exit(1)

        self.model.eval()
        ds_engine = deepspeed.init_inference(self.model,
                                 mp_size=1,
                                 dtype=torch.half,
                                 checkpoint=None,
                                 replace_with_kernel_inject=True)
        self.model = ds_engine.module

    def score(self, refs, outs, batch_size):
        scores_ls = []
        cur_data_dict = {'pivot': refs, 'mt': outs}
        cur_data_loader = preprocess_data(cur_data_dict, self.tokenizer, exp_config.max_length, batch_size,
                                          shuffle=False, sampler=False, mode='test')
        for batch in cur_data_loader:
            # generate a batch of ref, mt embeddings
            score = self.model(batch, 'last_layer').squeeze(1).tolist()
            scores_ls.extend(score)
        return scores_ls



from analysis import D2TStat

d2t_stat = D2TStat('D2T/BAGEL/sescore.pkl')

# Set valid metrics
valid_metrics = [
    #   'rouge1_f',
    #   'rouge2_f',
    #   'rougel_f',
    #   'bert_score_f',
    #   'mover_score',
    #   'prism_avg',
    # 'contra_score',
    'sescore',
    # 'bart_score_avg_f',
    # 'bart_score_para_avg_f',
    # 'bart_score_para_avg_f_de'
]

# The first argument is human metric while the latter is a list of metrics considered.
d2t_stat.evaluate_text('informativeness', valid_metrics)



import click
import torch
import json
from train.regression import *

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

@click.command()
@click.option('-load_benchmark', default=None, help='data for frank benchmark')
@click.option('-model_addr', default=None, help='locations of model weights')
def main(model_addr, load_benchmark):
    exp_config.device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    model = torch.load(model_addr).to(exp_config.device_id)
    print("model is loaded!")
    tokenizer = AutoTokenizer.from_pretrained("rembert-tok")
    benchmark_data = json.load(open(load_benchmark))

    # collect summary_outs and refs
    summary_outs = [ele['summary'] for ele in benchmark_data]
    refs = [ele['article'] for ele in benchmark_data]
    cur_data_dict = {'pivot': refs, 'mt': summary_outs}
    batch_size=16
    cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False, mode='test')
    scores_ls = []
    for batch in cur_data_loader:
        # generate a batch of ref, mt embeddings
        score = model(batch, 'last_layer').squeeze(1).tolist()
        scores_ls.extend(score)

    save_ls = []
    for bench_ele, score in zip(benchmark_data, scores_ls):
        temp_dict = {}
        temp_dict["hash"]=bench_ele["hash"]
        temp_dict["model_name"]=bench_ele["model_name"]
        temp_dict["article"]=bench_ele["article"]
        temp_dict["summary"]=bench_ele["summary"]
        temp_dict["score"]=score
        temp_dict["split"]=bench_ele["split"]
        save_ls.append(temp_dict)

    with open('frank_results.json', 'w') as f:
        json.dump(save_ls, f)

    print("File frank_results.json is saved!")

if __name__ == "__main__":
    main()



import json
from process_results import *
import itertools
import numpy as np
from tqdm.auto import tqdm
import click
import math

# key question: 
# why BLEURT, COMET and GPT3 all dropped around 6% in Kendall correlation, does this 6%
# shares some similarities in the mistakes or just coincidence
# /Users/wendaxu/opt/anaconda3/lib/python3.9/site-packages/mt_metrics_eval/stats.py

def Agreement(vect1, vect2, vect_names):
  """Compute pairwise agreement over two vectors, vect1 assumed to be gold."""
  agree, num_pairs, dis_ls, T, U, P, Q = 0, 0, [], 0, 0, 0, 0
  count, cor_count = 0, 0
  pairs_ls = list(itertools.combinations(zip(vect1, vect2, vect_names), 2))
  with tqdm(total=len(pairs_ls)) as pbar:
    for a, b in pairs_ls:
        pbar.update(1)
        if a[0] is None or b[0] is None:
            continue

        if (a[0] - b[0]) == 0:
            count += 1

        if np.sign(a[0] - b[0]) == np.sign(a[1] - b[1]):
            agree += 1
            if (a[1] - b[1]) == 0:
                cor_count+=1
        else:
            dis_ls += [(a[2], b[2])]

        if (a[0] - b[0] > 0 and a[1] - b[1] > 0) or (a[0] - b[0] < 0 and a[1] - b[1] < 0):
            P += 1
        elif (a[0] - b[0] > 0 and a[1] - b[1] < 0) or (a[0] - b[0] < 0 and a[1] - b[1] > 0):
            Q += 1
        else:
            if (a[0] - b[0]) == 0 and (a[1] - b[1]) != 0:
                T += 1
            elif (a[0] - b[0]) != 0 and (a[1] - b[1]) == 0:
                U += 1  
        num_pairs += 1  
    cor = (P-Q)/math.sqrt((P+Q+T)*(P+Q+U))
  
  print("Tie accuracy: ", cor_count/count)
  print("Total ties: ", count)
  print("tie accounts total: ", count/num_pairs)
  print("P: ", P)
  print("Q: ", Q)
  print("T: ", T)
  print("U: ", U)
  return agree, num_pairs, dis_ls, cor

@click.command()
@click.option('-wmt', default='wmt21.news')
@click.option('-lang', default='zh-en')
@click.option('-start', type=int)
@click.option('-end', type=int)
def main(wmt, lang, start, end):
    metric_names = ['comet', 'bleurt', 'gpt3', 'sescore2', 'bleu', 'bertscore', 'prism', 'yisi', 'comet-src-only-mqm', \
    'comet-mqm', 'chrf', 'ter']
    save_dict = {}
    for metric_name in metric_names:
        if metric_name == 'gpt3':
            scores_dict = json.load(open('wmt21.news_zh-en_0_300_gpt3.json'))
            scores_dict = {key: scores_dict[key][start:end] for key in scores_dict}
        elif metric_name == 'sescore2':
            sescore2_dict = json.load(open('baseline_results/zh-en_wmt21.news_contrascore.json'))
            scores_dict = {}
            for sys, scores_ls in sescore2_dict.items():
                scores_dict[sys] = []
                for index in indices_set:
                    scores_dict[sys] += [scores_ls[index]]
            scores_dict = {key: scores_dict[key][start:end] for key in scores_dict}
        else:
            if metric_name == 'comet':
                lines = open('baseline_results/COMET-DA_2020-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'bleurt':
                lines = open('baseline_results/bleurt-20-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'bleu':
                lines = open('baseline_results/sentBLEU-refB.seg.score', 'r').readlines()

            elif metric_name == 'bertscore':
                lines = open('baseline_results/BERTScore-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'prism':
                lines = open('baseline_results/Prism-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'yisi':
                lines = open('baseline_results/YiSi-1-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'comet-src-only-mqm':
                lines = open('baseline_results/COMET-QE-MQM_2021-src.seg.score', 'r').readlines()

            elif metric_name == 'comet-mqm':
                lines = open('baseline_results/COMET-MQM_2021-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'chrf':
                lines = open('baseline_results/chrF-refB.seg.score', 'r').readlines()
            
            elif metric_name == 'ter':
                lines = open('baseline_results/TER-refB.seg.score', 'r').readlines()

            else:
                print("metric name is not supported!")
                exit(1)

            evs = data.EvalSet(wmt, lang)
            gt_scores, indices_set, num_sys = process_gt_metric_outs(evs, start, end)

            pre_scores_dict = {}
            num_segs = int(len(lines)/num_sys)
            for line in lines:
                sys_name, score = line.split('\t')[0], float(line.split('\t')[1][:-1])
                if sys_name not in pre_scores_dict:
                    pre_scores_dict[sys_name] = []
                pre_scores_dict[sys_name]+=[score]
            scores_dict = {}
            for sys, scores_ls in pre_scores_dict.items():
                scores_dict[sys] = []
                for index in indices_set:
                    scores_dict[sys] += [scores_ls[index]]
            scores_dict = {key: scores_dict[key][start:end] for key in scores_dict}

        gt_scores_ls, baseline_ls, sys_index_ls = [], [], []
        sys_names = set(gt_scores) - {evs.std_ref} 

        for sys in sys_names:
            gt_scores_ls += gt_scores[sys]
            baseline_ls += scores_dict[sys]
            indices_ls = [sys+'_'+str(i) for i in range(len(gt_scores[sys]))]
            sys_index_ls += indices_ls

        agree, num_pairs, dis_ls, cor = Agreement(gt_scores_ls, baseline_ls, sys_index_ls)
        save_dict[metric_name] = dis_ls

        print("metric name: ", metric_name)
        print("Agree: ", agree)
        print("Disagree", len(dis_ls))
        print("Total: ", num_pairs)
        print("Accuracy: ", agree/num_pairs)
        print("Kendall Tau-Like: ", (agree-len(dis_ls))/num_pairs)
        print("Kendall: ", cor)
        print("-----------------------------------------")
        print()

    with open(f'metrics_{start}_{end}_mistake_pairs.json', 'w') as f:
        json.dump(save_dict, f)
    print("File is saved")

if __name__ == "__main__":
    main()  


class loss_funct():
    def __init__(self, strategy, emb_type, margin=0):
        self.strategy = strategy
        self.sim = Similarity(temp=config.temp)
        # max_length and hidden size are dataset specfic and model specfic (XLM-Roberta)
        if strategy == 'in_batch':
            self.loss_fct = nn.CrossEntropyLoss()
        elif strategy == 'mt_out_hard_one':
            self.loss_fct_1, self.loss_fct_2 = nn.MultiMarginLoss(margin=margin, reduction='mean'), nn.MultiMarginLoss(margin=margin*2, reduction='mean')
        else:
            print("We currently supports two strategies")
            exit(1)

    def loss_compute(self, train_batch, model, emb_type):
        src_pool_embed = pool(model, train_batch['input_ids'], train_batch['src_attn_masks'], emb_type)
        pos_pool_embed = pool(model, train_batch['tar'], train_batch['tar_attn_masks'], emb_type)

        if self.strategy == 'in_batch':
            cos_sim = self.sim(src_pool_embed.unsqueeze(1), pos_pool_embed.unsqueeze(0))
            # each label correspond to index of sample in the bacth, only the same index is the correct class
            labels = torch.arange(cos_sim.size(0)).long().to(config.device)
            loss = self.loss_fct(cos_sim, labels) # default is mean

        elif self.strategy == 'margin_src_ref_batch':
            pass
        elif self.strategy == 'margin_src_ref_mt1':
            pass

        elif self.strategy == 'margin_src_ref_mt1_batch':
            pass

        elif self.strategy == 'mt_out_hard_one':
            # debatable needs to further investigate
            mt_pool_embed = pool(model, train_batch['mt1'], train_batch['mt1_attn_masks'], emb_type)
            cos_sim_src_pos_batch_neg = self.sim(src_pool_embed.unsqueeze(1), pos_pool_embed.unsqueeze(0))
            cos_sim_hard_neg_batch_neg = self.sim(mt_pool_embed.unsqueeze(1), pos_pool_embed.unsqueeze(0))
            cos_sim_src_pos, cos_sim_src_hard_neg = torch.diag(cos_sim_src_pos_batch_neg, 0), self.sim(src_pool_embed, mt_pool_embed)
            cos_sim_pos_hard_neg_pair = torch.cat((cos_sim_src_pos.unsqueeze(1), cos_sim_src_hard_neg.unsqueeze(1)), dim=1)
            # pairwise ranking y labels between (src,ref,mt), (src, ref, neg), (mt, ref, neg)
            src_ref_mt_y = torch.tensor([0]*cos_sim_pos_hard_neg_pair.size(0)).to(config.device) # margin * 1
            src_ref_neg_y = torch.arange(cos_sim_src_pos_batch_neg.size(0)).to(config.device) # margin * 2
            mt_ref_neg_y = torch.arange(cos_sim_hard_neg_batch_neg.size(0)).to(config.device) # margin * 1
            # compute loss
            src_ref_mt_loss = self.loss_fct_1(cos_sim_pos_hard_neg_pair, src_ref_mt_y)
            src_ref_neg_loss = self.loss_fct_2(cos_sim_src_pos_batch_neg, src_ref_neg_y)
            mt_ref_neg_loss = self.loss_fct_1(cos_sim_hard_neg_batch_neg, mt_ref_neg_y)
            loss = 1/3*(src_ref_mt_loss + src_ref_neg_loss + mt_ref_neg_loss)
        return loss



from analysis import D2TStat
from utils import *

# tot_data = read_pickle('D2T/BAGEL/final_p.pkl')
# contra_data = read_pickle('D2T/BAGEL/all.pkl')
# for index, sub_dict in tot_data.items():
#     tot_data[index]['scores']['contra_score'] = contra_data[index]['scores']['contra_score']

# save_pickle(tot_data, 'D2T/final_p.pkl')

d2t_stat = D2TStat('D2T/final_p.pkl')

# Set valid metrics
valid_metrics = [
      'rouge1_f',
      'rouge2_f',
      'rougel_f',
      'bert_score_f',
      'mover_score',
      'prism_avg',
     'bart_score_avg_f',
     'contra_score',
]

# The first argument is human metric while the latter is a list of metrics considered.
d2t_stat.evaluate_text('naturalness', valid_metrics)
m1 = 'contra_score'
m2 = 'bart_score_avg_f'
result = d2t_stat.sig_test_two(m1, m2, 'naturalness')

if result == 1:
    print(f'{m1} is significantly better than {m2}')
elif result == -1:
    print(f'{m2} is significantly better than {m1}')
else:
    print('cannot decide')



import random
import click
"""This program has two purposes:
1) Simplify the task to purely edit distance ops
2) Verify if the previous pipeline is correct (compare two strings are actually similar)"""

"""python3 syn_data/select_pair.py -raw_addr train.en -raw_index_addr train_2M.en -index_addr thres_margin_sep15/thres_margin_0_5027365_value_part1.txt """

@click.command()
@click.option("-raw_addr")
@click.option("-raw_index_addr_0")
@click.option("-raw_index_addr_1")
@click.option("-raw_index_addr_2")
@click.option("-raw_index_addr_3")
@click.option("-index_addr")
# prepare two strings for each instance for edit distance ops
def main(raw_addr, raw_index_addr_0, raw_index_addr_1, raw_index_addr_2, raw_index_addr_3, index_addr):
    raw_lines = open(raw_addr, 'r').readlines()
    raw_index_lines = open(raw_index_addr_0, 'r').readlines()
    raw_index_lines_1 = open(raw_index_addr_1, 'r').readlines()
    raw_index_lines_2 = open(raw_index_addr_2, 'r').readlines()
    raw_index_lines_3 = open(raw_index_addr_3, 'r').readlines()
    raw_index_lines.extend(raw_index_lines_1)
    raw_index_lines.extend(raw_index_lines_2)
    raw_index_lines.extend(raw_index_lines_3)

    index_file = open(index_addr, 'r')
    # start_index = int(index_addr.split('.')[0].split('_')[-2])
    # end_index = int(index_addr.split('.')[0].split('_')[-1])

    # print("start index: ", start_index)
    # print("end index: ", end_index)
    start_index=0
    save_file = open(f'pair_edit_ops_data_en_ja.txt', 'w')

    for cur_index, index_line in enumerate(index_file):
        if index_line != '\n':
            index_ls = index_line[:-1].strip().split('\t')
            str1 = raw_lines[cur_index+start_index][:-1]
            tar_index = random.choice(index_ls)
            # print(index_ls)
            str2 = raw_index_lines[int(tar_index)]
            if str1.strip() == str2[:-1].strip():
                print("bugs!")
                print(str1)
                print(str2)
                exit(1)
            final_line = str(cur_index+start_index) + '\t' + str1 + '\t' + str2
            save_file.write(final_line)

    print("File is saved!")

if __name__ == "__main__":
    main()



import click
import math

@click.command()
@click.option('-file_name')
@click.option('-num_file', type=int)
def main(file_name, num_file):
    inp_lines = open(file_name, 'r').readlines()
    seg_size = math.ceil(len(inp_lines)/num_file)
    prefix = file_name.split('.')[0]
    for i in range(0, len(inp_lines), seg_size):
        start_index, end_index = i, min(i+seg_size, len(inp_lines))
        save_file = open(f"{prefix}_{start_index}_{end_index}.txt", 'w')
        save_file.writelines(inp_lines[start_index:end_index])
        print(f"{prefix}_{start_index}_{end_index}.txt is saved!")

if __name__ == "__main__":
    main()



from nltk.tokenize import word_tokenize
import click
import string
import nltk
import spacy
nltk.download('punkt')

"""python3 syn_data/del_severe_measure.py -src_addr mt_severe_data/zh_en/train_400k.zh \
-ref_addr mt_severe_data/zh_en/train_400k.en -opt_addr mt_severe_data/zh_en/del_sen_op_loc_1.txt \
-idf_addr idf_weights/wiki_tfidf_terms.csv -save_folder mt_severe_data/zh_en"""

def detect_punkt(words_ls):
    for word in words_ls:
        if word not in string.punctuation:
            return False
    return True

@click.command()
@click.option('-ref_addr')
@click.option('-opt_addr')
@click.option('-idf_addr')
@click.option('-save_folder')
@click.option('-lang')
def main(ref_addr, opt_addr, idf_addr, save_folder, lang):
    tgt_texts = open(ref_addr, 'r').readlines()
    ops_loc_texts = open(opt_addr, 'r').readlines()
    stop_lines = open(idf_addr, 'r').readlines()
    stop_lines = [line[:-1] for line in stop_lines]

    print("load in weights")

    saveFile = open(f'{save_folder}/score_'+opt_addr.split('/')[-1], 'w')
    punkt_minor, minor, severe = 0, 0, 0

    if lang == 'japanese':
        nlp = spacy.blank('ja')

    for ops_loc in ops_loc_texts:
        ops_ls = ops_loc[:-1].split('\t')
        sen_index, start_end_index = int(ops_ls[0]), ops_ls[2].split('_')
        if lang == 'japanese':
            tgt_words_ls = [word.text for word in nlp(tgt_texts[sen_index][:-1])]
        else:
            tgt_words_ls = word_tokenize(tgt_texts[sen_index][:-1])
        start_index, end_index = int(start_end_index[0]), int(start_end_index[1])

        weight_ls=[]
        for word in tgt_words_ls[start_index:end_index]:
            if word.lower() in stop_lines:
                weight_ls.append(0)
            else:
                if word not in string.punctuation:
                    weight_ls.append(2)

        if detect_punkt(tgt_words_ls[start_index:end_index]):
            new_line = ops_loc[:-1]+'\t'+''+'\t'+str(-0.1)+'\n'
            punkt_minor+=1
        else:
            if max(weight_ls) < 1:
                new_line = ops_loc[:-1]+'\t'+''+'\t'+str(-1)+'\n'
                minor+=1
            else:
                new_line = ops_loc[:-1]+'\t'+''+'\t'+str(-5)+'\n'
                severe+=1
        saveFile.write(new_line)
    print(punkt_minor)
    print(minor)
    print(severe)

if __name__ == "__main__":
    main()



"""This function is used to control the length of each span in the masked target sentences, currently mu is set to be 3.5"""
def poisson_dist_span(mask_len, mu=3.5):
    span_ls = []
    while mask_len > 0:
        random_span = poisson.rvs(mu, size=1, loc=0)
        if random_span > 0 and random_span < mask_len:
            r = random_span
        else:
            r = mask_len
        mask_len = mask_len - r
        span_ls.append(r.item())

    return span_ls

"""This function is used to construct masked data for target language sentences before they concatenated along with source sentences.
Mask ratio can be set to control the ratio of the mask tokens in the sentences (mask_ratio = 0.70). Return the concatenated input"""
def construct_mask_data(src_data, ref_data, mask_token, sep_token, mask_ratio=0.70):
    src_data, ref_data = [data[:-1] for data in src_data], [data[:-1] for data in ref_data]
    mask_inp_data_ls, mask_ref_data_ls = [], []
    for src_sen, ref_sen in zip(src_data, ref_data):
        ref_words_ls = word_tokenize(ref_sen)
        mask_len = int(len(ref_words_ls) * mask_ratio)
        span_ls = poisson_dist_span(mask_len, mu=3.5)
        pre_index_ls, start_index_ls, end_index_ls = [], [], []
        mask_inp_data = []
        for index, span_len in enumerate(span_ls):
            if len(end_index_ls) > 0:
                start_index = randrange(end_index_ls[-1], len(ref_words_ls)-sum(span_ls[index:])+1)
            else:
                # only set the start index lower bound at step 0, set it to be 0
                start_index = randrange(0, len(ref_words_ls)-sum(span_ls[index:])+1)
            end_index = start_index + span_len
            if index == 0:
                # only set the start index lower bound at step 0, set it to be 0
                pre_index_ls.append((0, start_index))
            else:
                pre_index_ls.append((end_index_ls[-1], start_index))
            start_index_ls.append(start_index)
            end_index_ls.append(end_index)
        for start, end in pre_index_ls:
            mask_inp_data += ref_words_ls[start:end]+[mask_token]
        # store the ground truth masked data and masked input
        for start, end in zip(start_index_ls, end_index_ls):
            mask_ref_data_ls += ref_words_ls[start:end] + ['<SEP>']
        mask_inp_data_ls.append(src_sen +f' {sep_token} '+ TreebankWordDetokenizer().detokenize(mask_inp_data))
    return mask_inp_data_ls, mask_ref_data_ls



from transformers import AutoTokenizer
import torch

def main():
    tok_checkpoint = 'mbart-large-50-many-to-one-mmt-tok'
    model_checkpoint = 'mt_infilling_weights_zh-en_2M_1660497817/epoch4.ckpt'

    torch.cuda.empty_cache()
    device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    tokenizer = AutoTokenizer.from_pretrained(tok_checkpoint)
    tokenizer.add_special_tokens({'additional_special_tokens': ['<sep>']})
    model = torch.load(model_checkpoint).to(device)
    model.eval()

    inp_text = "阿拉伯人身上流淌着一种专制强加给他们的自卑和软弱，正是这种自卑和软弱造成了绝望、愤怒、暴力和偏狭。 <sep> - Arab men and women have shed the sense of humiliation and inferiority that despotism imposed on them – and that fostered desperation, <mask>."
    data = tokenizer(inp_text, return_tensors='pt')
    translated_tokens = model.generate(data['input_ids'].to(device))
    batch_texts = tokenizer.batch_decode(translated_tokens, skip_special_tokens=True)
    print(batch_texts)

if __name__ == "__main__":
    main()



import click
from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer

"""code to run: python3 syn_data/prepare_mask_data_infilling.py -src_addr train_400k.zh -ref_addr train_400k.en -op_addr mt_sen_op_loc_1.txt -lang_dir zh_en"""

@click.command()
@click.option('-src_addr', type=str)
@click.option('-ref_addr', type=str)
@click.option('-op_addr', type=str)
@click.option('-lang_dir', type=str)
def main(src_addr, ref_addr, op_addr, lang_dir):
    src_lines = open(src_addr, 'r').readlines()
    ref_lines = open(ref_addr, 'r').readlines()
    op_lines = open(op_addr, 'r').readlines()

    src_lines = [line[:-1] for line in src_lines]
    ref_lines = [line[:-1] for line in ref_lines]
    op_lines = [line[:-1] for line in op_lines]

    saveFile = open(lang_dir+"_"+op_addr, 'w')

    for line in op_lines:
        line_ls = line.split('\t')
        sen_index, op, edit_locs = int(line_ls[0]), int(line_ls[1]), line_ls[2]
        words_ls = word_tokenize(ref_lines[sen_index])
        if op == 1:
            edit_locs_start = int(edit_locs)
            new_words_ls = words_ls[:edit_locs_start] + ['<mask>'] + words_ls[edit_locs_start:]
        elif op == 2:
            edit_los_ls = edit_locs.split('_')
            edit_locs_start, edit_locs_end = int(edit_los_ls[0]), int(edit_los_ls[1])
            new_words_ls = words_ls[:edit_locs_start] + ['<mask>'] + words_ls[edit_locs_end:]
        new_line = src_lines[sen_index]+' <sep> '+TreebankWordDetokenizer().detokenize(new_words_ls)
        saveFile.write(new_line+'\n')

if __name__ == "__main__":
    main()



import click
import pandas as pd
import numpy as np

@click.command()
@click.option('-f')
@click.option('-src_lang')
@click.option('-tar_lang')
def main(f, src_lang, tar_lang):
    srcFile, tarFile = open(f"news_comp_{src_lang}_{tar_lang}.{src_lang}", 'w'), open(f"news_comp_{src_lang}_{tar_lang}.{tar_lang}", 'w')
    tsvFile = pd.read_csv(f, '\t', header=None, on_bad_lines='skip')
    # replace all empty locations with nans and drop all rows contain nan
    tsvFile.replace('', np.nan, inplace=True)
    tsvFile = tsvFile.dropna()
    src_ls, ref_ls = list(tsvFile[1]), list(tsvFile[0])
    src_ls = [src.replace('\n','')+'\n' for src in src_ls]
    ref_ls = [ref.replace('\n','')+'\n' for ref in ref_ls]
    assert(len(src_ls)==len(ref_ls))
    srcFile.writelines(src_ls)
    tarFile.writelines(ref_ls)
    print("files are produced!")

if __name__ == "__main__":
    main()



import click
from nltk.tokenize import word_tokenize
from random import randrange
from nltk.tokenize.treebank import TreebankWordDetokenizer
from tqdm.auto import tqdm
import json

"""This file is able to prepare the mask data for large scale synthetic data generation. The code to run:
python3 syn_data/prepare_mask_data.py -prefix cycle1_zh_en_news_train -src_data_file news_data/zh-en/news_comp_zh_en.zh_train.txt -ref_data_file news_data/zh-en/news_comp_zh_en.en_train.txt -mask_ratio 0.15"""

def construct_mask_data(src_data, ref_data, sep_token, mask_token, mask_ratio=0.15):
    src_data, ref_data = [data[:-1] for data in src_data], [data[:-1] for data in ref_data]
    mask_inp_data_ls, mask_ref_data_ls, global_mask_loc_ls = [], [], []
    with tqdm(total=len(src_data)) as pbar:
        for sen_index, (src_sen, ref_sen) in enumerate(zip(src_data, ref_data)):
            ref_words_ls = word_tokenize(ref_sen)
            mask_len = int(len(ref_words_ls) * mask_ratio)
            pre_index_ls, start_index_ls, end_index_ls = [], [], []
            mask_inp_data = []
            for index in range(5):
                if len(end_index_ls) > 0:
                    start_index = randrange(end_index_ls[-1], len(ref_words_ls)-(5-index)*mask_len+1)
                else:
                    # only set the start index lower bound at step 0, set it to be 0
                    start_index = randrange(0, len(ref_words_ls)-(5-index)*mask_len+1)
                end_index = start_index + mask_len
                if index == 0:
                    # only set the start index lower bound at step 0, set it to be 0
                    pre_index_ls.append((0, start_index))
                else:
                    pre_index_ls.append((end_index_ls[-1], start_index))
                start_index_ls.append(start_index)
                end_index_ls.append(end_index)

            seg_mask_loc_ls = []
            for loc_index, (start_index, end_index) in enumerate(zip(start_index_ls, end_index_ls)):
                mask_ref_inp_data = ref_words_ls[:start_index] + [mask_token] + ref_words_ls[end_index:]
                mask_tar_data = ref_words_ls[start_index:end_index]
                mask_inp_data_ls.append(f'{sen_index}'+'\t'+f'{loc_index}'+'\t'+src_sen +f' {sep_token} '+ TreebankWordDetokenizer().detokenize(mask_ref_inp_data)+'\n')
                mask_ref_data_ls.append(f'{sen_index}'+'\t'+f'{loc_index}'+'\t'+TreebankWordDetokenizer().detokenize(mask_tar_data)+'\n')
                seg_mask_loc_ls.append([start_index, end_index])
            global_mask_loc_ls.append(seg_mask_loc_ls)
            pbar.update(1)
    return mask_inp_data_ls, mask_ref_data_ls, global_mask_loc_ls

@click.command()
@click.option('-prefix', type=str)
@click.option('-src_data_file', type=str)
@click.option('-ref_data_file', type=str)
@click.option('-mask_ratio', type=float, default=0.15)
def main(prefix, src_data_file, ref_data_file, mask_ratio):
    src_data = open(src_data_file, 'r').readlines()
    ref_data = open(ref_data_file, 'r').readlines()
    src_data, ref_data = [src[:-1] for src in src_data], [ref[:-1] for ref in ref_data]
    mask_inp_data_ls, mask_ref_data_ls, global_mask_loc_ls = construct_mask_data(src_data, ref_data, '<sep>', '<mask>', mask_ratio)
    save_src_mask_file = open(f'{prefix}_src_mask.txt', 'w')
    save_ref_mask_file = open(f'{prefix}_ref_mask.txt', 'w')
    # save all the prepared data
    save_src_mask_file.writelines(mask_inp_data_ls)
    save_ref_mask_file.writelines(mask_ref_data_ls)
    with open(f"{prefix}_start_end_locs.json", 'w') as outfile:
        json.dump(global_mask_loc_ls, outfile)

    print("All files are saved!")

if __name__ == "__main__":
    main()



from comet import load_from_checkpoint
import click
import glob

"""CUDA_VISIBLE_DEVICES=0 python3 test_sescore.py -model_path sescore_ckpt/zh_en/checkpoint/sescore_english.ckpt -ref_path /opt/tiger/contra_score/wmt22-metrics-inputs-v6/metrics_inputs/txt/generaltest2022/references/generaltest2022.zh-en.ref.refA.en -out_path /opt/tiger/contra_score/wmt22-metrics-inputs-v6/metrics_inputs/txt/generaltest2022/system_outputs/zh-en/ -ref_type refA"""

@click.command()
@click.option('-model_path')
@click.option('-ref_path')
@click.option('-out_path')
@click.option('-ref_type')
def main(model_path, ref_path, out_path, ref_type):
    model = load_from_checkpoint(model_path)
    ref_lines = open(ref_path, 'r').readlines()
    ref_lines = [line[:-1] for line in ref_lines]

    for file_name in glob.glob(out_path+'/*'):
        out_lines = open(file_name, 'r').readlines()
        out_lines = [line[:-1] for line in out_lines]

        dataset = [{"src": ref, "mt": out} for ref, out in zip(ref_lines, out_lines)]
        seg_scores, _ = model.predict(dataset, batch_size=16, gpus=1)
        seg_scores = [str(score)+'\n' for score in seg_scores]

        save_path = file_name+f'.sescores_{ref_type}'
        savefile = open(save_path, 'w')
        savefile.writelines(seg_scores)

        print(f"File {save_path} is saved!")

if __name__ == "__main__":
    main()



import click
import torch
import torch.nn as nn
import wandb
from torch.utils.data import DataLoader
from datasets import Dataset
from transformers import (
    XLMRobertaTokenizerFast,
    AutoModelForMaskedLM,
    DataCollatorWithPadding,
    get_linear_schedule_with_warmup,
    AdamW
)
import math
from torch.nn.parallel import DistributedDataParallel as DDP
import torch.distributed as dist
import os
import random
from mt_metrics_eval import data
import time

# 1) obtain the triplet pairs (ranking difference and use it to determine margins)
# 2) use margin ranking loss for all triplets, based on rankings

human_mapping_dict = {
    "wmt21.news": {
        'en-de': ['refA', 'refD'],
        'en-ru': ['refB'],
        'zh-en': ['refA']
    },
    "wmt20": {
        'en-de': 'refb',
        'zh-en': 'refb'
    }
}

class exp_config():
    max_length = 256
    hidden_size = 1024
    temp = 0.1
    lr = 3e-05

class Similarity(nn.Module):
    """
    Dot product or cosine similarity
    """

    def __init__(self, temp):
        super().__init__()
        self.temp = temp
        self.cos = nn.CosineSimilarity(dim=-1)

    def forward(self, x, y):
        return self.cos(x, y) / self.temp

def sent_emb(hidden_states, emb_type, attention_mask):
    if emb_type == 'last_layer':
        sen_embed = (hidden_states[-1]*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    elif emb_type == 'avg_first_last':
        sen_embed = ((hidden_states[-1]+hidden_states[0])/2.0*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    else:
        print(f"{emb_type} sentence emb type is not supported!")
        exit(1)
    return sen_embed

def pool(model, encoded_input, attention_mask, emb_type):
    encoded_input, attention_mask = encoded_input.to(exp_config.device_id), attention_mask.to(exp_config.device_id)
    outputs = model(input_ids=encoded_input, attention_mask=attention_mask, output_hidden_states=True)
    pool_embed = sent_emb(outputs.hidden_states, emb_type, attention_mask)
    return pool_embed

class Stratified_CL_XLM_Roberta(nn.Module):
    def __init__(self, model_addr):
        super().__init__()
        self.xlm = AutoModelForMaskedLM.from_pretrained(model_addr)

    def forward(self, batch, emb_type, src_based, eval_mode):
        pivot_pool_embed = pool(self.xlm, batch['input_ids'], batch['pivot_attn_masks'], emb_type)
        if eval_mode:
            pos_pool_embed = pool(self.xlm, batch['pos'], batch['pos_attn_masks'], emb_type)
            neg_pool_embed = pool(self.xlm, batch['neg'], batch['neg_attn_masks'], emb_type)
            return pivot_pool_embed, pos_pool_embed, neg_pool_embed
        else:
            if src_based:
                pos_pool_embed = pool(self.xlm, batch['pos'], batch['pos_attn_masks'], emb_type)
            else:
                # positive sample embedding is generated through dropout rate 0.1
                pos_pool_embed = pool(self.xlm, batch['input_ids'], batch['pivot_attn_masks'], emb_type)
            num_bacth_neg, num_attn_neg_masks = torch.transpose(batch['neg'], 0, 1), torch.transpose(batch['neg_attn_masks'], 0, 1)
            batch_pos_neg_ls = [pos_pool_embed]
            # compute one neg embedding for each batch per time
            for batch_neg, batch_attn_neg_masks in zip(num_bacth_neg, num_attn_neg_masks):
                ele_neg_pool_embed = pool(self.xlm, batch_neg, batch_attn_neg_masks, emb_type)
                batch_pos_neg_ls.append(ele_neg_pool_embed)
            pos_neg_pool_embed = torch.stack(batch_pos_neg_ls, 0)
            pos_neg_pool_embed = torch.transpose(pos_neg_pool_embed, 0, 1)
            return pivot_pool_embed, pos_neg_pool_embed

def preprocess_data(triplets_score_dict, tokenizer, max_length, batch_size, src_based, shuffle=True, sampler=True, eval_mode=False):
    if src_based:
        ds = Dataset.from_dict({"pivot": triplets_score_dict['pivot'], 'pos': triplets_score_dict['pos'], \
        'neg': triplets_score_dict['neg']})
    else:
        ds = Dataset.from_dict({"pivot": triplets_score_dict['pivot'], 'neg': triplets_score_dict['neg']})

    def preprocess_function(examples):
        model_inputs = {}
        # pivot examples added into dataloader, one pivot per instance
        pivot = tokenizer(examples['pivot'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['input_ids'], model_inputs['pivot_attn_masks'] = pivot['input_ids'], pivot['attention_mask']
        # only src based examples need postive instance
        if src_based:
            # pos examples added into dataloader, one pos per instance
            pos = tokenizer(examples['pos'], max_length=max_length, padding='max_length', truncation=True)
            model_inputs['pos'], model_inputs['pos_attn_masks'] = pos["input_ids"], pos['attention_mask']
        if eval_mode:
            # in eval mode, one neg per instance
            neg = tokenizer(examples['neg'], max_length=max_length, padding='max_length', truncation=True)
            model_inputs['neg'], model_inputs['neg_attn_masks'] = neg["input_ids"], neg['attention_mask']
        else:
            # neg examples added into dataloader, k negs per instance
            neg_ls, attns_ls = [], []
            for result in examples['neg']:
                neg = tokenizer(result, max_length=max_length, padding='max_length', truncation=True)
                neg_ls.append(neg["input_ids"])
                attns_ls.append(neg['attention_mask'])
            model_inputs['neg'], model_inputs['neg_attn_masks'] = neg_ls, attns_ls
        return model_inputs

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors = 'pt'
    )

    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

class loss_funct():
    def __init__(self, score_type):
        # be cautiously when applying cosine because order of pos neg will be opposite (targets=1)
        if score_type == "cosine":
            self.score = Similarity(temp=exp_config.temp)
            self.margin = 0.01
            self.loss_fct = torch.nn.CrossEntropyLoss()
        # targets = -1 when applying L2 distance
        elif score_type == "L2":
            self.score = nn.PairwiseDistance(p=2)
            self.margin = 1
            self.loss_fct = torch.nn.MarginRankingLoss(margin=self.margin)
        else:
            print("We only support two distance metrics: cosine and L2")
            exit(1)

    # If src_based, ref will be used as pos samples (pos:0, batchNeg: 6)->max_diff=6. If ref_based, max_diff=5
    def loss_compute(self, batch, model, emb_type, src_based):
        pivot_pool_embed, pos_neg_pool_embed = model(batch, emb_type, src_based=src_based, eval_mode=False)
        batch_scores = self.score(pivot_pool_embed.unsqueeze(1), pivot_pool_embed.unsqueeze(0))
        n = batch_scores.size(0)
        # remove the diagonal of scores
        batch_scores = batch_scores.flatten()[1:].view(n-1, n+1)[:,:-1].reshape(n, n-1)
        hard_scores = self.score(pivot_pool_embed.unsqueeze(1), pos_neg_pool_embed)
        scores = torch.cat((hard_scores, batch_scores), dim=1)
        labels = torch.zeros(scores.size(0)).long().to(exp_config.device_id)
        return self.loss_fct(scores, labels)

def baselines_cl_eval(srcs, mt_outs_dict, refs, emb_type, model, batch_size, score, tokenizer):
    with torch.no_grad():
        # load tokenizer and models, already specified addr for tokenizer
        mt_scores_dict = {'src': {}, 'ref': {}, 'src_ref': {}}
        score_funct = Similarity(temp=1) if score=='cosine' else nn.PairwiseDistance(p=2)
        # generate src embeddings
        for mt_name, mt_outs in mt_outs_dict.items():
            for key in mt_scores_dict:
                mt_scores_dict[key][mt_name] = []

            cur_data_dict = {'pivot': srcs, 'pos': refs, 'neg': mt_outs}
            cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, 200, shuffle=False, sampler=False, src_based=True, eval_mode=True)
            for batch in cur_data_loader:
                # generate a batch of src, ref, mt embeddings
                src_pool_embeds, ref_pool_embeds, mt_pool_embeds = model(batch, emb_type, src_based=True, eval_mode=True)
                # the score is computed between (src and mt) and (ref and mt)
                src_ls, ref_ls = score_funct(src_pool_embeds, mt_pool_embeds).tolist(), score_funct(ref_pool_embeds, mt_pool_embeds).tolist()
                h_mean_ls = [2*src_score*ref_score/(src_score+ref_score) for src_score, ref_score in zip(src_ls, ref_ls)]
                if score == 'L2':
                    src_ls = [1/(1+score) for score in src_ls]
                    ref_ls = [1/(1+score) for score in ref_ls]
                    h_mean_ls = [1/(1+score) for score in h_mean_ls]
                mt_scores_dict['src'][mt_name].extend(src_ls)
                mt_scores_dict['ref'][mt_name].extend(ref_ls)
                mt_scores_dict['src_ref'][mt_name].extend(h_mean_ls)

        return mt_scores_dict

def store_cl_loss(model, loss_manager, train_batch, emb_type, src_based):
    print("start to eval loss")
    with torch.no_grad():
        save_dict = {}
        train_loss = loss_manager.loss_compute(train_batch, model, emb_type, src_based)
        save_dict['total_train_loss'] = train_loss.item()
    return save_dict

def store_corr_eval(evs, mt_scores_dict, mode, wmt, lang):
    # process the ground truth human ratings
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)
    qm_human = qm_no_human.copy()
    qm_human.update(human_mapping_dict[wmt][lang])
    save_dict, temp_ls = {}, []

    for eval_type, scores_dict in mt_scores_dict.items():
        if mode == 'sys':
            # compute system-level scores (overwrite) otherwise seg scores are available already
            scores_dict = {mt_name: [sum(scores)/len(scores)] for mt_name, scores in scores_dict.items()}
        mqm_bp = evs.Correlation(mqm_scores, scores_dict, qm_human)
        mqm_bp_no = evs.Correlation(mqm_scores, scores_dict, qm_no_human)

        if mode == 'seg':
            save_dict[f'{eval_type}_seg_system_human']=mqm_bp.Kendall()[0]
            save_dict[f'{eval_type}_seg_system']=mqm_bp_no.Kendall()[0]
            temp_ls.append(mqm_bp.Kendall()[0])
        elif mode == 'sys':
            save_dict[f'{eval_type}_sys_system_human']=mqm_bp.Pearson()[0]
            save_dict[f'{eval_type}_sys_system']=mqm_bp_no.Pearson()[0]
            temp_ls.append(mqm_bp.Pearson()[0])
        else:
            print('Please choose between seg and sys!')
            exit(1)
    return max(temp_ls), save_dict


@click.command()
@click.option('-src_based', type=bool, help="if not specify or false means ref-based", default=False)
@click.option('-lang_dir', type=str, help="en_de or zh_en", default="zh_en")
@click.option('-gradient_accumulation_steps', default=1, type=int)
@click.option('-batch_size', type=int, help="train and eval batch size for contrastive learning", default=12)
@click.option('-emb_type', type=str, help="choose from last_layer, avg_first_last and states_concat", default="last_layer")
@click.option('-num_epoch', type=int, help="Number of epoches to train", default=5)
@click.option('-eval_step', type=int, help="Number of steps to evaluate", default=400)
@click.option('-num_warmup_steps', type=int, help="Number of steps to warm up", default=0)
@click.option('-save_dir_name', type=str, help="the dir name of weights being saved", default=None)
@click.option('-score_type', type=str, help="choose between L2 and cosine", default="cosine")
@click.option('-enable_loss_eval', type=bool, help="If given true, we will evaluate loss per eval step", default=True)
@click.option('-pivot_file', type=str, help="wiki_raw_2M.en", default="zh_en_2000000_train.txt")
@click.option('-pos_file', type=str, help="Optioanl: src will be pivot and target will be pos", default=None)
@click.option('-neg_file', type=str, help="wiki_cl_2M_5_zh_en.txt", default="mt_cl_5_zh_en.txt")
def main(src_based, lang_dir, gradient_accumulation_steps, batch_size, emb_type, num_epoch, eval_step, \
            num_warmup_steps, save_dir_name, score_type, enable_loss_eval, pivot_file, pos_file, neg_file):
    # load in eval data
    wmt, lang = 'wmt21.news', lang_dir.replace('_', '-')
    evs = data.EvalSet(wmt, lang)
    srcs, mt_outs_dict, refs = evs.src, evs.sys_outputs, evs.all_refs[evs.std_ref]
    # initalize the process
    dist.init_process_group(backend="nccl")
    rank = int(os.environ['LOCAL_RANK'])
    # only main process initalize wandb
    if rank == 0:
        # initalize the project parameters into Wandb, store experiment specific parameters
        wandb.init(project="ContraScore", config=
        {
            "src_based": src_based,
            "strategy": "Stratified CL",
            "epoch": num_epoch,
            "eval_step": eval_step,
            "emb_type": emb_type,
            "train batch size": batch_size * gradient_accumulation_steps * 8,
            "margin": 1 if score_type == 'L2' else 0.01,
            "score_type": score_type
        })

    exp_config.device_id = rank % torch.cuda.device_count()
    # set cuda device with rank and clear ram cache to ensure balanced ram allocations
    torch.cuda.set_device(rank)
    torch.cuda.empty_cache()
    # we will build bilingual sentence embedding models. XLM for both en-de and zh-en
    tokenizer = XLMRobertaTokenizerFast.from_pretrained("xlm-roberta-large-tok")
    if emb_type == 'last_layer':
        model = Stratified_CL_XLM_Roberta("xlm-roberta-large-model").to(exp_config.device_id)
    else:
        print("Incorrect model type!")
        exit(1)

    # parallelize the pipeline into multiple gpus
    optimizer = AdamW(model.parameters(), lr=exp_config.lr)
    raw_neg_lines = open(neg_file, 'r').readlines()
    neg_samples = [ele[:-1].split('\t') for ele in raw_neg_lines]

    def file_to_dict(file_type):
        raw_pivot_lines = open(pivot_file, 'r').readlines()
        pivot_samples = [ele[:-1] for ele in raw_pivot_lines]
        if src_based:
            raw_pos_lines = open(pos_file, 'r').readlines()
            pos_samples = [ele[:-1] for ele in raw_pos_lines]
            triplets_score_dict = {'pivot': pivot_samples, 'pos': pos_samples, 'neg': neg_samples}
        else:
            triplets_score_dict = {'pivot': pivot_samples, 'neg': neg_samples}
        return triplets_score_dict

    triplets_score_train_dict = file_to_dict('train')
    train_dataloader = preprocess_data(triplets_score_train_dict, tokenizer, exp_config.max_length, batch_size, shuffle=True, sampler=True, \
        src_based=False, eval_mode=False)

    model = DDP(model, device_ids=[exp_config.device_id], find_unused_parameters=True)
    model.train()

    max_train_steps = math.ceil(len(train_dataloader) / gradient_accumulation_steps) * num_epoch
    lr_scheduler = get_linear_schedule_with_warmup(
        optimizer=optimizer,
        num_warmup_steps=num_warmup_steps,
        num_training_steps=max_train_steps,
    )

    loss_manager = loss_funct(score_type)

    # save at end of epoch and at main processls
    if not os.path.isdir(f'{save_dir_name}') and rank == 0:
        os.makedirs(f'{save_dir_name}')

    with torch.autograd.set_detect_anomaly(True):
        for epoch in range(num_epoch):
            # reset the best correlation each time at start of epoch
            cur_best_cor = float('-inf')
            torch.cuda.empty_cache() # empty cache in gpus
            train_dataloader.sampler.set_epoch(epoch) # set the sampler at each epoch
            for step, train_batch in enumerate(train_dataloader):
                # evaluate at every eval_step and also at the end of epoch (includes the beginning loss)
                if ((step % (eval_step * gradient_accumulation_steps) == 0) or (step == len(train_dataloader) - 1)) and rank == 0:
                    # store all the losses in wandb
                    print("start to evaluate!")
                    model.eval()
                    wandb_temp_dict = {}
                    # evaluate on the seg and sys correlations
                    start_eval_time = time.time()
                    mt_scores_dict = baselines_cl_eval(srcs, mt_outs_dict, refs, emb_type, model, batch_size, score_type, tokenizer)
                    step_seg_cor, save_seg_dict = store_corr_eval(evs, mt_scores_dict, 'seg', wmt, lang)
                    _, save_sys_dict = store_corr_eval(evs, mt_scores_dict, 'sys', wmt, lang)
                    wandb_temp_dict.update(save_seg_dict)
                    wandb_temp_dict.update(save_sys_dict)
                    # monitor the training loss
                    if enable_loss_eval:
                        save_loss_dict = store_cl_loss(model, loss_manager, train_batch, emb_type, src_based=src_based)
                        wandb_temp_dict.update(save_loss_dict)
                    wandb.log(wandb_temp_dict)
                    print("Testing Duration: ", time.time()-start_eval_time)
                    # save at the best epoch step
                    if step_seg_cor > cur_best_cor:
                        cur_best_cor=step_seg_cor
                        torch.save(model.module, f'{save_dir_name}/epoch{epoch}_best.ckpt')
                        print(f"Saved best model at current epoch {epoch}!")
                    model.train()
                    torch.cuda.empty_cache()

                train_loss = loss_manager.loss_compute(train_batch, model, emb_type, src_based)

                # accumulate losses at weights, each is normalized by accumulation steps
                train_loss = train_loss / gradient_accumulation_steps
                train_loss.backward()

                if step % gradient_accumulation_steps == 0 or step == len(train_dataloader) - 1:
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad() # clear the grads

if __name__ == "__main__":
    random.seed(10)
    main()



import random
import click
import glob
import random

"""Offer the same data splits for src, ref and mt files"""
"""python3 data_split.py -file1 news_data/en-de/news_comp_en_de.en -file2 news_data/en-de/news_comp_en_de.de -file3 merge_outputs/en_XX_de_DE_total.txt"""

@click.command()
@click.option('-file1')
@click.option('-file2')
@click.option('-file3')
def main(file1, file2, file3):
    line1_ls = open(file1, 'r').readlines()
    line2_ls = open(file2, 'r').readlines()
    line3_ls = open(file3, 'r').readlines()

    ls_temp = list(zip(line1_ls, line2_ls, line3_ls))
    random.shuffle(ls_temp)

    new_line1_ls, new_line2_ls, new_line3_ls = zip(*ls_temp)
    new_line1_ls, new_line2_ls, new_line3_ls = list(new_line1_ls), list(new_line2_ls), list(new_line3_ls)

    train_file1 = open(file1+'_train.txt', 'w')
    train_file2 = open(file2+'_train.txt', 'w')
    train_file3 = open(file3+'_train.txt', 'w')

    dev_file1 = open(file1+'_dev.txt', 'w')
    dev_file2 = open(file2+'_dev.txt', 'w')
    dev_file3 = open(file3+'_dev.txt', 'w')

    split_index = int(len(new_line1_ls)*0.9)

    train_file1.writelines(new_line1_ls[:split_index])
    train_file2.writelines(new_line2_ls[:split_index])
    train_file3.writelines(new_line3_ls[:split_index])

    dev_file1.writelines(new_line1_ls[split_index:])
    dev_file2.writelines(new_line2_ls[split_index:])
    dev_file3.writelines(new_line3_ls[split_index:])

    print("Three new files are reproduced!")

if __name__ == "__main__":
    main()



import pandas as pd
import click

data_2017 = pd.read_csv('da_data/2017-da.csv')
data_2018 = pd.read_csv('da_data/2018-da.csv')
data_2019 = pd.read_csv('da_data/2019-da.csv')
data = pd.concat([data_2017, data_2018, data_2019], ignore_index=True)

@click.command()
@click.option('-lp', help="choose either en-de or zh-en")
def main(lp):
    extracted_data = data.loc[data['lp'] == lp].dropna()
    save_file = open(f'{lp}_da.txt', 'w')
    for ref, mt, score in zip(extracted_data['ref'], extracted_data['mt'], extracted_data['raw_score']):
        normalized_score = (score-100)/4
        # print(ref)
        # print(mt)
        # print(normalized_score)
        # print()
        save_file.write(ref+'\t'+mt+'\t'+str(normalized_score)+'\n')
    print(f'{lp}_da.txt is saved!')

if __name__ == "__main__":
    main()



import click

@click.command()
@click.option('-id1')
@click.option('-id2')
def main(id1, id2):
    id1_lines = open(id1, 'r')
    id2_lines = open(id2, 'r')
    start_index, end_index = id1.split('_')[2], id1.split('_')[3]

    new_id_file = open(f'merge_thres_margin_{start_index}_{end_index}.txt', 'w')

    for id1_line, id2_line in zip(id1_lines, id2_lines):
        new_id_line = id1_line[:-1]+'\t'+id2_line
        new_id_file.write(new_id_line)

    print("File is saved!")

if __name__ == "__main__":
    main()



import pandas as pd
import json
import numpy as np

data = pd.read_excel('英德_中英翻译模型评估-英语.xlsx', header=None, \
                   sheet_name=['中-英全数据-1','中-英全数据-2', '中-英全数据-3'])

ori_data = pd.read_excel('annotated_zh_en_bio_med_1000.xlsx')

annotations = ['中-英全数据-1','中-英全数据-2', '中-英全数据-3']
src_ls, out_ls, scores_ls, ref_ls, ele_src_ls, ele_ref_ls = [], [], [], [], [], []

load_refs_ls = open('medline_zh2en_en.txt', 'r').readlines()
load_srcs_ls = open('medline_zh2en_zh.txt', 'r').readlines()

refs_dict = {}
for line in load_refs_ls:
    doc_id, seg_id, txt = line.split('\t')[0], line.split('\t')[1], line.split('\t')[2][:-1]
    refs_dict[doc_id+"_"+seg_id] = txt

for line in load_srcs_ls[:100]:
    doc_id, seg_id, txt = line.split('\t')[0], line.split('\t')[1], line.split('\t')[2][:-1]
    ele_src_ls += [txt]
    ele_ref_ls += [refs_dict[doc_id+"_"+seg_id]]

for i in range(10):
    ref_ls += ele_ref_ls
    src_ls += ele_src_ls

out_ls = list(ori_data['translation'])

scores_arr = 0
# from 1 to 4: 1->src, 2->output, 3->score, 4->annotations   
for anno in annotations:
    cur_ls = list(data[anno][3])[:-1]
    print(np.isnan(np.array(cur_ls).any()))
    scores_arr += np.array(cur_ls)
scores_ls = list(scores_arr/3)

print(src_ls[90:100])
print(ref_ls[90:100])

bio_gt_dict = {}
bio_gt_dict['src'], bio_gt_dict['ref'], bio_gt_dict['out'], bio_gt_dict['score'] = src_ls, ref_ls, out_ls, scores_ls

with open(f'gt_zh-en_biomed.json', 'w') as f:
    json.dump(bio_gt_dict, f)

print("File is saved!")


import click
import torch
import json
from train.regression import *

# python3 inference/inference_regression.py -wmt wmt21.news -lang zh-en -model_addr epoch0_best_3616_zhen_pos_neg_sep20.ckpt
def baselines_cl_eval(mt_outs_dict, refs, emb_type, model, batch_size, tokenizer):
    with torch.no_grad():
        # load tokenizer and models, already specified addr for tokenizer
        mt_scores_dict = {}
        # generate src embeddings
        for mt_name, mt_outs in mt_outs_dict.items():
            mt_scores_dict[mt_name] = []
            cur_data_dict = {'pivot': refs, 'mt': mt_outs}
            cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False, mode='test')
            for batch in cur_data_loader:
                # generate a batch of ref, mt embeddings
                score = model(batch, emb_type).squeeze(1).tolist()
                mt_scores_dict[mt_name].extend(score)
        return mt_scores_dict

def store_corr_eval(evs, mt_scores_dict, mode, wmt, lang):
    # process the ground truth human ratings
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)
    qm_human = qm_no_human.copy()
    if wmt != 'wmt21.tedtalks':
        qm_human.update(human_mapping_dict[wmt][lang])

    if mode == 'sys':
        # compute system-level scores (overwrite) otherwise seg scores are available already
        mt_scores_dict = {mt_name: [sum(scores)/len(scores)] for mt_name, scores in mt_scores_dict.items()}
    mqm_bp = evs.Correlation(mqm_scores, mt_scores_dict, qm_human)
    mqm_bp_no = evs.Correlation(mqm_scores, mt_scores_dict, qm_no_human)

    if mode == 'seg':
        print("seg_system_human: ", mqm_bp.Kendall()[0])
        print("seg_system: ", mqm_bp_no.Kendall()[0])
    elif mode == 'sys':
        print("sys_system_human: ", mqm_bp.Pearson()[0])
        print("sys_system: ", mqm_bp_no.Pearson()[0])
    else:
        print('Please choose between seg and sys!')
        exit(1)

@click.command()
@click.option('-wmt')
@click.option('-lang')
@click.option('-load_file', default=None)
@click.option('-model_base')
@click.option('-model_addr', default=None, help='locations of model weights')
def main(lang, wmt, model_addr, load_file, model_base):
    evs = data.EvalSet(wmt, lang)
    mt_outs_dict, refs = evs.sys_outputs, evs.all_refs[evs.std_ref]
    exp_config.device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    if not load_file:
        model = torch.load(model_addr).to(exp_config.device_id)
        model.eval()
        print("model is loaded!")
        tokenizer = AutoTokenizer.from_pretrained(f"{model_base}-tok")
        mt_scores_dict = baselines_cl_eval(mt_outs_dict, refs, 'last_layer', model, 200, tokenizer)
        with open(f'{lang}_{wmt}.json', 'w') as f:
            json.dump(mt_scores_dict, f)
        print("result is saved!")
    else:
        mt_scores_dict = json.load(open(load_file))
    store_corr_eval(evs, mt_scores_dict, 'seg', wmt, lang)
    store_corr_eval(evs, mt_scores_dict, 'sys', wmt, lang)

if __name__ == "__main__":
    main()



import pandas as pd
from nltk.tokenize import word_tokenize

language, severity = 'german', 'Major'
print(language)
print(severity)
if severity == 'Minor':
    thres = -1
elif severity == 'Major':
    thres = -5
else:
    print("Wrong severity measures!")

if language == 'english':
    df_span = pd.read_csv('wmt-mqm-human-evaluation/newstest2020/zhen/mqm_newstest2020_zhen.tsv', delimiter='\t')
    df_score = pd.read_csv('wmt-mqm-human-evaluation/newstest2020/zhen/mqm_newstest2020_zhen.avg_seg_scores.tsv', delimiter='\t')
    save_addr="severity_analysis/zh_en"
elif language == 'german':
    df_span = pd.read_csv('wmt-mqm-human-evaluation/newstest2020/ende/mqm_newstest2020_ende.tsv', delimiter='\t')
    df_score = pd.read_csv('wmt-mqm-human-evaluation/newstest2020/ende/mqm_newstest2020_ende.avg_seg_scores.tsv', delimiter='\t')
    save_addr="severity_analysis/en_de"

# save into three files
src_file = open(f'{save_addr}/{severity}/{language}_{severity}_2020_src_text.txt', 'w')
tar_file = open(f'{save_addr}/{severity}/{language}_{severity}_2020_tar_text.txt', 'w')
loc_file = open(f'{save_addr}/{severity}/{language}_{severity}_mined_sen_op_loc.txt', 'w')
cont_file = open(f'{save_addr}/{severity}/{language}_{severity}_mined_cont.txt', 'w')

examples = []

for line in df_score['system mqm_avg_score seg_id']:
    line_ls = line.split()
    if float(line.split()[1]) == thres:
        examples.append([line_ls[0], line_ls[2]])

sen_index,count=0,0
# extract all examples
for example in examples:
    # match with id, system and severity
    span_df = df_span.loc[(df_span['system'] == example[0]) & (df_span['seg_id'] == int(example[1])) \
    & (df_span['severity'] == severity)]
    if len(list(span_df['target']))>0:
        # all operations will be annotated as 2 (replace)
        for src_ele, tar_ele in zip(span_df['source'], span_df['target']):
            if '<v>' in tar_ele:
                original_sen = tar_ele.replace('<v>', ' ').replace('</v>', ' ')
                original_sen = original_sen.strip()
                src_sen = src_ele.replace('<v>', '' ).replace('</v>', ' ')
                src_sen = src_sen.strip()
                start_index, span_len = len(word_tokenize(tar_ele.split('<v>')[0])), len(word_tokenize(tar_ele.split('<v>')[1].split('</v>')[0]))
                cont = tar_ele.split('<v>')[1].split('</v>')[0]
                if word_tokenize(cont) == word_tokenize(original_sen)[start_index:start_index+span_len]:
                    src_file.write(src_sen+'\n')
                    tar_file.write(original_sen+'\n')
                    loc_file.write(str(sen_index)+'\t'+str(2)+'\t'+str(start_index)+"_"+str(start_index+span_len)+'\n')
                    cont_file.write(cont+'\n')
                    sen_index+=1
                else:
                    print(word_tokenize(cont))
                    print(word_tokenize(original_sen)[start_index:start_index+span_len])
                    print()
                    count+=1
print(count)
print("All files are generated!")



from mt_metrics_eval import data

human_mapping_dict = {
    "wmt21.news": {
        'en-de': ['refA'],
        'en-ru': ['refB'],
        'zh-en': ['refA']
    },
    "wmt20": {
        'en-de': ['refb'],
        'zh-en': ['refb']
    },
}

wmt, lang = 'wmt21.news', 'en-de'
evs = data.EvalSet(wmt, lang)

mqm_scores = evs.Scores('seg', 'mqm')
wmt_scores = evs.Scores('seg', 'wmt-z')

new_index_set = set()
for sys in wmt_scores:
    if sys in mqm_scores:
        index_set = set()
        for temp_index, (ele_mqm, ele_wmt) in enumerate(zip(mqm_scores[sys], wmt_scores[sys])):
            if ele_mqm != None and ele_wmt != None:
                index_set.add(temp_index)

        if len(new_index_set) == 0:
            new_index_set.update(index_set)
        else:
            new_index_set = new_index_set & index_set

print(new_index_set)
new_wmt_scores, new_mqm_scores = {}, {}
for sys in wmt_scores:
    if sys in mqm_scores:
        temp_wmt_ls, temp_mqm_ls = [], []
        for index in sorted(new_index_set):
            temp_wmt_ls.append(wmt_scores[sys][index])
            temp_mqm_ls.append(mqm_scores[sys][index])
        new_wmt_scores[sys]=[sum(temp_wmt_ls)/len(temp_wmt_ls)]
        new_mqm_scores[sys]=[sum(temp_mqm_ls)/len(temp_mqm_ls)]

qm_no_human = set(new_mqm_scores) - set(evs.all_refs)
qm_human = qm_no_human.copy()
qm_human.update(human_mapping_dict[wmt][lang])

print(new_mqm_scores)
print(new_wmt_scores)
mqm_bp = evs.Correlation(new_mqm_scores, new_wmt_scores, qm_human & set(new_wmt_scores))
mqm_bp_no = evs.Correlation(new_mqm_scores, new_wmt_scores, qm_no_human & set(new_wmt_scores))

print(mqm_bp.Kendall()[0])
print(mqm_bp_no.Kendall()[0])
print()
print(mqm_bp.Pearson()[0])
print(mqm_bp_no.Pearson()[0])



from datasets import load_dataset
import click
import os


@click.command()
@click.option('-lang')
def main(lang):
    load_dataset('mc4', lang)
    print("dataset is downloaded!")

if __name__ == "__main__":
    main()



import torch
import click
from transformers import (
    AutoTokenizer,
    DataCollatorWithPadding,
)
from torch.utils.data import DataLoader
import math
from datasets import Dataset
import os

"""python3 syn_data/biimt_gen_data.py -prefix mt_infilling_data -batch_size 64 -model_checkpoint mt_infilling_weights_zh-en_2M_1660497817/epoch4.ckpt
-tok_checkpoint mbart-large-50-many-to-one-mmt-tok -src_file zh-en-mask.txt"""

"""Yield batch sized list of indices."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

def preprocess_data(inp_data, tokenizer, max_length, batch_size, shuffle=True, sampler=True):
    ds = Dataset.from_dict({"src": inp_data})
    # generate input data
    def preprocess_function(examples):
        model_inputs = {}
        srcs = tokenizer(examples['src'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['input_ids'], model_inputs['src_attn_masks'] = srcs['input_ids'], srcs['attention_mask']
        return model_inputs

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors='pt'
    )

    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

@click.command()
@click.option('-batch_size', type=int, default=80)
@click.option('-model_checkpoint', type=str)
@click.option('-tok_checkpoint', type=str, default="mbart-large-50-many-to-one-mmt-tok")
@click.option('-src_file', type=str, help="addr to the src file", default='')
@click.option('-save_dir', type=str)
@click.option('-prefix', type=str)
@click.option('-part_index', type=int, help="from 0 to 7")
def main(batch_size, model_checkpoint, tok_checkpoint, src_file, save_dir, prefix, part_index):
    torch.cuda.empty_cache()
    device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    tokenizer = AutoTokenizer.from_pretrained(tok_checkpoint)
    num_added_toks = tokenizer.add_special_tokens({'additional_special_tokens': ['<sep>']})
    model = torch.load(model_checkpoint).to(device)
    model.eval()
    # load in dev data
    mask_inp = open(src_file, 'r').readlines()
    mask_inp = [sen[:-1] for sen in mask_inp]
    file_loc_index = list(range(0, len(mask_inp), math.ceil(len(mask_inp)/8)))+[len(mask_inp)]
    start_index, end_index = file_loc_index[part_index], file_loc_index[part_index+1]
    dataloader = preprocess_data(mask_inp[start_index:end_index], tokenizer, 256, batch_size, shuffle=False, sampler=False)

    gen_file = open(f'{save_dir}/{prefix}_{start_index}_{end_index}.outputs', 'w')
    with torch.no_grad():
        for data in dataloader:
            translated_tokens = model.generate(data['input_ids'].to(device))
            batch_texts = tokenizer.batch_decode(translated_tokens, skip_special_tokens=True)
            for text in batch_texts:
                gen_file.write(text+'\n')
    print(f"All outputs are saved at {prefix}.outputs")

if __name__ == "__main__":
    main()



import re
from datasets import load_dataset
from nltk.tokenize import sent_tokenize
import nltk
import click
nltk.download('punkt')

lang_mapping = {
    'de': 'german',
    'en': 'english'
}

@click.command()
@click.option('-lang')
def main(lang):
    dataset = load_dataset("wikipedia", f"20220301.{lang}")
    saveFile = open(f'wiki_raw.{lang}', 'w')
    for ele in dataset['train']:
        article = ele['text']
        sentences_ls = re.split('\n\n|\n|\t\t|\t', article)
        for sentence in sentences_ls:
            if len(sentence.split())>10:
                for sent in sent_tokenize(sentence, language=lang_mapping[lang]):
                    saveFile.write(sent+'\n')
    print("All texts are processed!")

if __name__ == "__main__":
    main()



import click

"""Example code to run:
python3 syn_data/stats_analysis.py -lang_dir zh_en -out_addr syn_data_gen_zh_en_train_1660639001/cycle1_infill_mt_0.15_zh_en_total.txt"""

def empty_lines_stats(addr):
    count = 0
    inFile = open(addr, 'r')
    for line in inFile:
        if line=='\n':
            count+=1
    print(count)

def rep_dectec_rewrite(lang_dir, out_addr):
    src_file = open(f'news_prepare_mask_data_syn_gen/cycle1_{lang_dir}_news_train_src_mask.txt', 'r')
    ref_file = open(f'news_prepare_mask_data_syn_gen/cycle1_{lang_dir}_news_train_ref_mask.txt', 'r')
    out_file = open(out_addr, 'r')
    count, total, index_dict = 0, 0, {}
    for src_line, ref_line, out_line in zip(src_file, ref_file, out_file):
        ref_content = ref_line.split('\t')[-1]
        if ref_content == out_line:
            count+=1
        else:
            index = ref_line.split('\t')[0]
            if index not in index_dict:
                index_dict[index]=0
            index_dict[index]+=1
        total+=1

    index_count_1, index_count_2, index_count_3, index_count_4, index_count_5 = 0,0,0,0,0
    for index, index_count in index_dict.items():
        if index_count==1:
            index_count_1+=1
        elif index_count==2:
            index_count_2+=1
        elif index_count==3:
            index_count_3+=1
        elif index_count==4:
            index_count_4+=1
        else:
            assert(index_count==5)
            index_count_5+=1

    print("Number of lines which repeat: ", count)
    print("Number of lines which not repeat: ", total-count)
    print("Intances contain one error: ", index_count_1)
    print("Intances contain two errors: ", index_count_2)
    print("Intances contain three errors: ", index_count_3)
    print("Intances contain four error: ", index_count_4)
    print("Intances contain five error: ", index_count_5)
    total_instances = index_count_1+index_count_2+index_count_3+index_count_4+index_count_5
    print("Total number of instances: ", total_instances)

@click.command()
@click.option('-lang_dir')
@click.option('-out_addr')
def main(lang_dir, out_addr):
    rep_dectec_rewrite(lang_dir, out_addr)

if __name__ == "__main__":
    main()



import click
from mt_metrics_eval import data
import json
import wandb
from inference import *

# WANDB_NAME=margin_src_ref_mt1_batch_last_layer_L2_zh_en_sys python3 inference/log_scores_wandb.py -model_addr new_margin_loss_weights/margin_src_ref_mt1_batch_last_layer_L2_weights_1659948126 -mode sys -wmt wmt21.news -lang zh-en
# WMT21 en-de src, ref, src_ref based System-level Correlation With Human

def corr_eval_wandb(evs, mt_scores_dict, mode, wmt, lang, model_addr):
    # process the ground truth human ratings
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)
    qm_human = qm_no_human.copy()
    qm_human.update(human_mapping_dict[wmt][lang])

    final_dict = {}
    for eval_type, scores_dict in mt_scores_dict.items():
        print(f"Use {eval_type} at {mode} for metric evaluation:")
        if mode == 'sys':
            # compute system-level scores (overwrite) otherwise seg scores are available already
            scores_dict = {mt_name: [sum(scores)/len(scores)] for mt_name, scores in scores_dict.items()}
        mqm_bp = evs.Correlation(mqm_scores, scores_dict, qm_human)
        mqm_bp_no = evs.Correlation(mqm_scores, scores_dict, qm_no_human)

        if mode == 'seg':
            final_dict[eval_type]=[mqm_bp.Kendall()[0], mqm_bp_no.Kendall()[0]]
        elif mode == 'sys':
            final_dict[eval_type]=[mqm_bp.Pearson()[0], mqm_bp_no.Pearson()[0]]
        else:
            print("your mode is wrong!")
            exit(1)

    wandb.log({
        f"src_{wmt}_{lang}_{mode}_{model_addr}_with_human": final_dict['src'][0],
        f"src_{wmt}_{lang}_{mode}_{model_addr}_without_human": final_dict['src'][1],
        f"ref_{wmt}_{lang}_{mode}_{model_addr}_with_human": final_dict['ref'][0],
        f"ref_{wmt}_{lang}_{mode}_{model_addr}_without_human": final_dict['ref'][1],
        f"src_ref_{wmt}_{lang}_{mode}_{model_addr}_with_human": final_dict['src_ref'][0],
        f"src_ref_{wmt}_{lang}_{mode}_{model_addr}_without_human": final_dict['src_ref'][1],
    })

@click.command()
@click.option('-model_addr')
@click.option('-mode')
@click.option('-wmt', default="wmt21.news")
@click.option('-lang', default="en-de")
def main(model_addr, mode, wmt, lang):
    wandb.init(project="ContraScore", config=
    {
        'mode': mode,
        'model_type': model_addr,
        'type': "eval_score_mode"
    })

    evs = data.EvalSet('wmt21.news', lang)
    for i in range(5):
        mt_scores_dict = json.load(open(f'{model_addr}/epoch{i}.ckpt.json'))
        print(f"load in {model_addr}/epoch{i}.ckpt.json")

        corr_eval_wandb(evs, mt_scores_dict, mode, wmt, lang, model_addr.split('/')[1])

if __name__ == "__main__":
    main()



import glob
import pandas as pd

files_ls = sorted(glob.glob('annotated_outputs/*'))
src_lines = open('src.de', 'r').readlines()
src_lines = [line[:-1] for line in src_lines]*10
tot_lines = []

for file_name in files_ls:
    cur_lines = open(file_name, 'r').readlines()
    cur_lines = [line[:-1] for line in cur_lines]
    tot_lines += cur_lines

df_dict = {'source': src_lines, 'translation': tot_lines, 'score': ['']*1000, 'error_type': ['']*1000}
df = pd.DataFrame.from_dict(df_dict)
df.to_excel("annotated_en_de_1000.xlsx")

print("file is generated!") 



import click
from tqdm.auto import tqdm

"""code to run:
python3 syn_data/margin_compute.py -id_addr ids_query_5027365_10054730_value_part1.txt \
-dist_addr dists_query_5027365_10054730_value_part1.txt -sum_addr_1 sum_query_0_5027365_value_part1.txt \
-sum_addr_2 sum_query_5027365_10054730_value_part1.txt -sum_addr_3 sum_query_10054730_15082095_value_part1.txt \
-sum_addr_4 sum_query_15082095_20109460_value_part1.txt
"""

@click.command()
@click.option('-id_addr', help="ids_query_5027365_10054730_value_part1.txt")
@click.option('-dist_addr', help="dists_query_5027365_10054730_value_part1.txt")
@click.option('-sum_addr_1', help="sum_query_0_5027365_value_part1.txt")
@click.option('-sum_addr_2', help="sum_query_5027365_10054730_value_part1.txt", default=None)
@click.option('-sum_addr_3', help="sum_query_10054730_15082095_value_part1.txt", default=None)
@click.option('-sum_addr_4', help="sum_query_15082095_20109460_value_part1.txt", default=None)
@click.option('-thres', help="top k candidates are computed for margin criterion, ex: 128", type=int)
def main(id_addr, dist_addr, sum_addr_1, sum_addr_2, sum_addr_3, sum_addr_4, thres):
    ids_lines = open(id_addr, 'r')
    print("load in one file")
    dist_lines = open(dist_addr, 'r')
    print("load in two files")
    sum_lines = open(sum_addr_1, 'r').readlines()
    if sum_addr_2:
        sum_lines_2 = open(sum_addr_2, 'r').readlines()
        sum_lines.extend(sum_lines_2)
    if sum_addr_3:
        sum_lines_3 = open(sum_addr_3, 'r').readlines()
        sum_lines.extend(sum_lines_3)
    if sum_addr_4:
        sum_lines_4 = open(sum_addr_4, 'r').readlines()
        sum_lines.extend(sum_lines_4)

    print("loaded all the files")

    save_ids_file = open(f'thres_{thres}_sorted_'+id_addr.split('/')[1], 'w')
    save_dist_file = open(f'thres_{thres}_sorted_'+dist_addr.split('/')[1], 'w')

    id_sum_dict, k = {}, thres
    for index, sum_val in enumerate(sum_lines):
        id_sum_dict[index] = float(sum_val[:-1])

    with tqdm(total=5027365) as pbar:
        for sen_index, (id_ls, dist_ls) in enumerate(zip(ids_lines, dist_lines)):
            id_ls, dist_ls = id_ls[:-1].split('\t'), dist_ls[:-1].split('\t')
            id_ls = [int(id) for id in id_ls]
            dist_ls = [float(dist) for dist in dist_ls]
            forward_sum = id_sum_dict[sen_index]
            margin_ls = []
            for id, dist in zip(id_ls[:thres], dist_ls[:thres]):
                backward_sum = id_sum_dict[id]
                margin = dist/(forward_sum/(2*k)+backward_sum/(2*k))
                margin_ls.append(margin)
            sorted_pair_ls = sorted(zip(margin_ls, id_ls[:thres]), reverse=True) # dec order
            sorted_margin_ls = [str(ele[0]) for ele in sorted_pair_ls]
            sorted_id_ls = [str(ele[1]) for ele in sorted_pair_ls]

            save_ids_file.write('\t'.join(sorted_id_ls)+'\n')
            save_dist_file.write('\t'.join(sorted_margin_ls)+'\n')
            pbar.update(1)

    print("Files are saved!")

if __name__ == "__main__":
    main()



from tqdm.auto import tqdm
import click

"""python3 syn_data/extract_neg.py -dists_addr sorted_dists_query_20109460_25136824_value_part2.txt -ids_addr sorted_ids_query_20109460_25136824_value_part2.txt -query_addr train.en -val_addr news_wiki.en"""

@click.command()
@click.option('-dists_addr')
@click.option('-ids_addr')
@click.option('-query_addr')
@click.option('-val_addr')
@click.option('-val_addr_1')
@click.option('-val_addr_2')
@click.option('-val_addr_3')
def main(dists_addr, ids_addr, query_addr, val_addr, val_addr_1, val_addr_2, val_addr_3):
    dist_file = open(dists_addr, 'r')
    id_file = open(ids_addr, 'r')
    query_lines = open(query_addr, 'r').readlines()

    val_lines = open(val_addr, 'r').readlines()
    val_lines_1 = open(val_addr_1, 'r').readlines()
    val_lines_2 = open(val_addr_2, 'r').readlines()
    val_lines_3 = open(val_addr_3, 'r').readlines()

    val_lines.extend(val_lines_1)
    val_lines.extend(val_lines_2)
    val_lines.extend(val_lines_3)

    start_index, end_index = int(ids_addr.split('_')[-4]), ids_addr.split('_')[-3]
    thres_id_file = open(f'thres_margin_{start_index}_{end_index}_value_part{ids_addr[-5]}.txt', 'w')
    min_samples = 128
    unqualified = 0
    correct = 0

    # with tqdm(total=5027365) as pbar:
    for cur_index, (dist_line, id_line) in enumerate(zip(dist_file, id_file)):
        dist_ls = dist_line[:-1].split('\t')
        id_ls = id_line[:-1].split('\t')
        cur_id_ls = []

        for id, dist in zip(id_ls, dist_ls):
            if float(dist) >= 1.06 and val_lines[int(id)].strip() != query_lines[start_index+cur_index].strip():
                cur_id_ls.append(id)

        thres_id_file.write('\t'.join(cur_id_ls)+'\n')
        if cur_index == int(id_ls[0]):
            correct+=1
        if len(cur_id_ls) == 0:
            unqualified+=1
        min_samples=min(min_samples, len(cur_id_ls))
            # pbar.update(1)

    print(correct)
    print(unqualified)
    print("File is generated!")

if __name__ == "__main__":
    main()



import click
import string
from nltk.tokenize import word_tokenize
import nltk
nltk.download('punkt')

def detect_punkt(words_ls):
    for word in words_ls:
        if word not in string.punctuation:
            return False
    return True

@click.command()
@click.option('-ref_addr')
@click.option('-file_name')
@click.option('-save_folder')
@click.option('-language', help="english or german")
@click.option('-thres', type=float)
def main(ref_addr, file_name, save_folder, language, thres):
    ref_lines = open(ref_addr, 'r').readlines()
    ref_lines = [ref[:-1] for ref in ref_lines]
    lines = open(file_name, 'r').readlines()
    lines = [line[:-1] for line in lines]
    saveFile = open(f'{save_folder}/score_'+file_name.split('/')[-1], 'w')

    punkt_minor, minor, severe = 0, 0, 0
    for line in lines:
        ops_ls = line[:-1].split('\t')
        sen_index, op_type = int(ops_ls[0]), int(ops_ls[1])
        if op_type == 1:
            if detect_punkt(word_tokenize(ops_ls[3], language)):
                score = -0.1
                punkt_minor+=1
            else:
                if float(line.split('\t')[-1]) >= thres:
                    score = -1
                    minor+=1
                else:
                    score = -5
                    severe+=1
        elif op_type == 2:
            start_index, end_index = int(ops_ls[2].split('_')[0]), int(ops_ls[2].split('_')[1])
            if detect_punkt(word_tokenize(ref_lines[sen_index])[start_index:end_index]) and detect_punkt(word_tokenize(ops_ls[3])):
                score = -0.1
                punkt_minor+=1
            else:
                if float(line.split('\t')[-1]) >= thres:
                    score = -1
                    minor+=1
                else:
                    score = -5
                    severe+=1
        else:
            print("wrong type!")
            exit(1)

        prefix = '\t'.join(line.split('\t')[:-1])
        new_line = prefix+'\t'+str(score)+'\n'
        saveFile.write(new_line)

    print(punkt_minor)
    print(minor)
    print(severe)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

with open("README.md", "r", encoding="utf-8-sig") as fh:
    long_description = fh.read()

packages = \
['sescore2',
 'sescore2.inference',
 'sescore2.preprocess',
 'sescore2.syn_data',
 'sescore2.train'
 ]

package_data = \
{'': ['*']}

install_requires = \
['transformers',
'accelerate==0.20.3',
'sentencepiece',
'protobuf==3.20.*',
'python-snappy',
'pandas',
'nvitop',
'click',
'datasets',
'wandb',
'scipy',
'absl-py',
'torch',
'torchvision',
'torchaudio',
'deepspeed']

setup_kwargs = {
    'name': 'sescore2',
    'version': '1.0.7',
    'description': 'SESCORE2: Learning Text Generation Evaluation via Synthesizing Realistic Mistakes',
    'long_description': long_description,
    'long_description_content_type': "text/markdown",
    'author': 'Wenda Xu, Xian Qian, Mingxuan Wang, Lei Li, William Yang Wang',
    'author_email': 'wendaxu@ucsb.edu',
    'maintainer': 'Wenda Xu, Zihan Ma',
    'maintainer_email': 'zihan_ma@ucsb.edu',
    'url': 'https://github.com/xu1998hz/SEScore2_archive',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)



from datasets import load_dataset

sen_pair_dict = {}

test_dataset = load_dataset('paws', 'labeled_final')

for sen_1, sen_2, label in zip(test_dataset['test']['sentence1'],  \
    test_dataset['test']['sentence2'], test_dataset['test']['label']):
    if sen_1 not in sen_pair_dict:
        sen_pair_dict[sen_1] = {}
        sen_pair_dict[sen_1][0] = []
        sen_pair_dict[sen_1][1] = []
    sen_pair_dict[sen_1][label].append(sen_2)

better_ls, worse_ls, ref_ls = [], [], []
for sen1, cur_dict in sen_pair_dict.items():
    if len(cur_dict[0])>0 and len(cur_dict[1])>0:
        better_ls.append(cur_dict[1][0]+'\n')
        worse_ls.append(cur_dict[0][0]+'\n')
        ref_ls.append(sen1+'\n')

save_better = open('paws_better.txt', 'w')
save_worse = open('paws_worse.txt', 'w')
save_ref = open('paws_ref.txt', 'w')

save_better.writelines(better_ls)
save_worse.writelines(worse_ls)
save_ref.writelines(ref_ls)
print("All files are saved!")



import click
import sys
sys.path.insert(0, '/home/wendaxu/SEScore2')
# from nltk.tokenize import word_tokenize
import nltk
from train.regression import *
import json
import pandas as pd
# import evaluate
# nltk.download('punkt')

# speech translation:
# 1) baseline: CUDA_VISIBLE_DEVICES=0 python3 inference/test_all_metrics.py -metric_name comet-da-20 -benchmark_name st-en-ja
# 2) sescore2: CUDA_VISIBLE_DEVICES=0 python3 inference/test_all_metrics.py -metric_name sescore2 -benchmark_name st-en-ja -ckpt ckpt -model_base google/rembert

# CUDA_VISIBLE_DEVICES=0 python3 inference/test_all_metrics.py -metric_name sescore2 -benchmark_name x-en-wmt20-da -ckpt epoch0_best_en_de_rembert.ckpt -model_base rembert -lp
def test_save_metric_benchmark(metric_name, benchmark_name, ref_lines, out_lines, lang, ckpt, model_base=None, tot_src_ls=None):
    if metric_name == 'comet-da-20':
        from comet import download_model, load_from_checkpoint
        model_path = download_model("wmt20-comet-da")
        model = load_from_checkpoint(model_path)
        if benchmark_name == 'st-en-ja':
            src_lines = open('mqm_en_ja/mqm_en_ja_src.txt', 'r').readlines()
            src_lines = [line[:-1] for line in src_lines]
            dataset = [{"src": src, "ref": ref, "mt": out} for src, ref, out in zip(src_lines, ref_lines, out_lines)]
            seg_scores, _ = model.predict(dataset, batch_size=16, gpus=1)
        elif benchmark_name == 'mt-de-en':
            src_lines = open('de_en_benchmark/en_de.src', 'r').readlines()
            src_lines = [line[:-1] for line in src_lines]
            dataset = [{"src": src, "ref": ref, "mt": out} for src, ref, out in zip(src_lines, ref_lines, out_lines)]
            seg_scores, _ = model.predict(dataset, batch_size=16, gpus=1)
        elif benchmark_name == 'x-en-wmt20-da':
            src_lines = tot_src_ls
            dataset = [{"src": src, "ref": ref, "mt": out} for src, ref, out in zip(src_lines, ref_lines, out_lines)]
            seg_scores, _ = model.predict(dataset, batch_size=16, gpus=1)
        else:
            print("Your current benchmark doesn't support comet!")
            exit(1)
    elif metric_name == 'bleurt-20':
        from bleurt import score
        checkpoint = "bleurt/BLEURT-20"
        scorer = score.BleurtScorer(checkpoint)
        seg_scores = scorer.score(references=ref_lines, candidates=out_lines)
    elif metric_name == 'sent-bleu':
        seg_scores=[]
        sacrebleu = evaluate.load("sacrebleu")
        for out_line, ref_line in zip(out_lines, ref_lines):
            if lang == 'ja':
                results = sacrebleu.compute(predictions=[out_line], references=[[ref_line]], tokenize='ja-mecab')
            else:
                results = sacrebleu.compute(predictions=[out_line], references=[[ref_line]])
            seg_scores += [results["score"]]
    elif metric_name == 'chrf':
        seg_scores=[]
        chrf = evaluate.load("chrf")
        for ref, hyp in zip(ref_lines, out_lines):
            results = chrf.compute(predictions=[hyp], references=[[ref]])
            seg_scores+=[results['score']]
    elif metric_name == 'ter':
        seg_scores=[]
        ter = evaluate.load("ter")
        for ref, hyp in zip(ref_lines, out_lines):
            if lang == 'ja':
                results = ter.compute(predictions=[hyp], references=[[ref]], normalized=True, support_zh_ja_chars=True)
            else:
                results = ter.compute(predictions=[hyp], references=[[ref]], case_sensitive=True)
            seg_scores+=[results['score']]
    elif metric_name == 'prism':
        import os
        from prism import Prism
        prism = Prism(model_dir=os.environ['MODEL_DIR'], lang=lang)
        seg_scores=prism.score(cand=out_lines, ref=ref_lines, segment_scores=True)
    elif metric_name == 'bartscore':
        from bart_score import BARTScorer
        bart_scorer = BARTScorer(device='cuda:0', checkpoint='facebook/bart-large')
        seg_scores=bart_scorer.score(out_lines, ref_lines, batch_size=16)
    elif metric_name == 'bertscore':
        from bert_score import score
        seg_scores=[]
        if lang != 'en':
            bertscore_lang = 'others'
        else:
            bertscore_lang = 'en'
        _, _, F = score(out_lines, ref_lines, lang=bertscore_lang)
        seg_scores=F.tolist()
    elif metric_name == 'sescore':
        from comet import load_from_checkpoint
        if lang == 'en':
            model = load_from_checkpoint('sescore_ckpt/zh_en/checkpoint/english.ckpt')
        elif lang == 'de':
            model = load_from_checkpoint('sescore_ckpt/en_de/checkpoint/sescore_german.ckpt')
        else:
            print("SEScore only supports en and de!")
        dataset = [{"src": ref, "mt": out} for ref, out in zip(ref_lines, out_lines)]
        seg_scores, _ = model.predict(dataset, batch_size=16, gpus=1)
    elif metric_name == 'sescore2':
        from SEScore2 import SEScore2
        scorer = SEScore2('en') # load in metric with specified language, en (English), de (German), ja ('Japanese')
        exp_config.device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        scores_ls = scorer.score(ref_lines, out_lines, 16)
    else:
        print("Your current metric name is not supported!")
        exit(1)

    return seg_scores

def s2t_en_ja_benchmark(seg_scores, gt_lines):
    from mt_metrics_eval import data
    evs = data.EvalSet('wmt21.news', 'zh-en')
    mt_score_dict, gt_score_dict = {}, {}

    mt_score_dict['sys1'] = seg_scores[:78]
    mt_score_dict['sys2'] = seg_scores[78:78*2]
    mt_score_dict['sys3'] = seg_scores[78*2:78*3]
    mt_score_dict['sys4'] = seg_scores[78*3:]

    gt_score_dict['sys1'] = gt_lines[:78]
    gt_score_dict['sys2'] = gt_lines[78:78*2]
    gt_score_dict['sys3'] = gt_lines[78*2:78*3]
    gt_score_dict['sys4'] = gt_lines[78*3:]

    mqm_bp = evs.Correlation(gt_score_dict, mt_score_dict, mt_score_dict.keys())
    print(mqm_bp.Kendall())

def mt_en_de_benchmark(seg_scores, gt_lines):
    from mt_metrics_eval import data
    evs = data.EvalSet('wmt21.news', 'zh-en')
    mt_score_dict, gt_score_dict = {}, {}

    for index, i in enumerate(range(0, 700, 100)):
        mt_score_dict[f'sys{index}'] = seg_scores[i:i+100]
        gt_score_dict[f'sys{index}'] = gt_lines[i:i+100]

    mqm_bp = evs.Correlation(gt_score_dict, mt_score_dict, mt_score_dict.keys())
    print(mqm_bp.Kendall())

@click.command()
@click.option('-metric_name')
@click.option('-benchmark_name')
@click.option('-ckpt', help="only used to sescore2")
@click.option('-load_file', type=bool, default=False)
@click.option('-file_first')
@click.option('-file_sec')
@click.option('-caption_file')
@click.option('-model_base')
@click.option('-lp')
@click.option('-lang', help="use en, ja or de")
def main(metric_name, benchmark_name, load_file, ckpt, file_first, file_sec, caption_file, model_base, lp, lang):
    if load_file:
        if benchmark_name == 'st-en-ja':
            gt_lines = open('mqm_en_ja/mqm_en_ja_scores.txt', 'r').readlines()
            gt_lines = [float(line[:-1]) for line in gt_lines]
            seg_scores = open(file_first, 'r').readlines()
            seg_scores = [float(score[:-1]) for score in seg_scores]
            s2t_en_ja_benchmark(seg_scores, gt_lines)
        elif benchmark_name == 'mt-de-en':
            gt_lines = open('de_en_benchmark/en_de_tot.scores', 'r').readlines()
            gt_lines = [float(line[:-1]) for line in gt_lines]
            seg_scores = open(file_first, 'r').readlines()
            seg_scores = [float(score[:-1]) for score in seg_scores]
            mt_en_de_benchmark(seg_scores, gt_lines)
        elif benchmark_name == 'paws':
            better = open(file_first, 'r').readlines()
            better = [float(ele[:-1]) for ele in better]
            worse = open(file_sec, 'r').readlines()
            worse = [float(ele[:-1]) for ele in worse]
            import numpy as np
            diff = np.array(better)-np.array(worse)
            concordant, total = np.sum(diff>0), len(better)
            discordant = total-concordant
            print(concordant)
            print((concordant-discordant)/total)
        elif benchmark_name == 'webnlg2020':
            import numpy as np
            model_dicts = json.load(open(file_first))
            gt_dicts = json.load(open(file_sec))
            gt_dicts_cor = {sys: list(gt_dicts[sys]['Correctness'].values()) for sys in gt_dicts}
            gt_dicts_cov = {sys: list(gt_dicts[sys]['DataCoverage'].values()) for sys in gt_dicts}
            gt_dicts_flu = {sys: list(gt_dicts[sys]['Fluency'].values()) for sys in gt_dicts}
            gt_dicts_rel = {sys: list(gt_dicts[sys]['Relevance'].values()) for sys in gt_dicts}
            gt_dicts_str = {sys: list(gt_dicts[sys]['TextStructure'].values()) for sys in gt_dicts}

            gt_dicts_tot = {sys: (np.array(list(gt_dicts[sys]['Correctness'].values())) + \
            np.array(list(gt_dicts[sys]['DataCoverage'].values())) + \
            np.array(list(gt_dicts[sys]['Fluency'].values())) + np.array(list(gt_dicts[sys]['Relevance'].values())) + \
            np.array(list(gt_dicts[sys]['TextStructure'].values())))/5 for sys in gt_dicts}

            gt_dicts_tot = {sys: gt_dicts_tot[sys].tolist() for sys in gt_dicts_tot}

            from mt_metrics_eval import data
            evs = data.EvalSet('wmt21.news', 'zh-en')
            mqm_bp = evs.Correlation(gt_dicts_cor, model_dicts, model_dicts.keys())
            print("Correctness: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gt_dicts_cov, model_dicts, model_dicts.keys())
            print("Coverage: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gt_dicts_flu, model_dicts, model_dicts.keys())
            print("Fluency: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gt_dicts_rel, model_dicts, model_dicts.keys())
            print("Relevance: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gt_dicts_str, model_dicts, model_dicts.keys())
            print("Text Stucture: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gt_dicts_tot, model_dicts, model_dicts.keys())
            print("Total: ", mqm_bp.Kendall())
        elif benchmark_name == 'webnlg':
            model_dicts = json.load(open(file_first))
            new_model_dicts, flu_gt_dicts, gra_gt_dicts, sem_gt_dicts = {}, {}, {}, {}
            df = pd.read_csv('./webnlg-human-evaluation/all_data_final_averaged.csv')
            for team, seg_score_dict in model_dicts.items():
                new_model_dicts[team] = [seg_score_dict[str(i)] for i in range(223)]
                flu_gt_dicts[team] = list(df.loc[df['team'] == team]['fluency'])
                gra_gt_dicts[team] = list(df.loc[df['team'] == team]['grammar'])
                sem_gt_dicts[team] = list(df.loc[df['team'] == team]['semantics'])
            from mt_metrics_eval import data
            evs = data.EvalSet('wmt21.news', 'zh-en')
            mqm_bp = evs.Correlation(flu_gt_dicts, new_model_dicts, new_model_dicts.keys())
            print("fluency: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(gra_gt_dicts, new_model_dicts, new_model_dicts.keys())
            print("gra: ", mqm_bp.Kendall())
            mqm_bp = evs.Correlation(sem_gt_dicts, new_model_dicts, new_model_dicts.keys())
            print("sem: ", mqm_bp.Kendall())
    else:
        if benchmark_name == 'st-en-ja':
            ref_lines = open('mqm_en_ja/mqm_en_ja_ref.txt', 'r').readlines()
            ref_lines = [line[:-1] for line in ref_lines]
            out_lines = open('mqm_en_ja/mqm_en_ja_sys.txt', 'r').readlines()
            out_lines = [line[:-1] for line in out_lines]
            save_name = f'{benchmark_name}_{metric_name}_results.txt'
            save_file = open(save_name, 'w')
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_lines, out_lines, lang, ckpt, model_base)
            seg_scores = [str(score)+'\n' for score in seg_scores]
            save_file.writelines(seg_scores)
            print("file is saved!")
        elif benchmark_name == 'mt-zh-en-bio-med':
            data = json.load(open('gt_zh-en_biomed.json'))
            save_name = f'{benchmark_name}_{metric_name}_results.txt'
            save_file = open(save_name, 'w')
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, data['ref'], data['out'], lang, ckpt, model_base)
            seg_scores = [str(score)+'\n' for score in seg_scores]
            save_file.writelines(seg_scores)
            print("file is saved!")
        elif benchmark_name == 'mt-de-en':
            ref_lines = open('de_en_benchmark/en_de.ref', 'r').readlines()
            ref_lines = [line[:-1] for line in ref_lines]
            out_lines = open('de_en_benchmark/en_de.out', 'r').readlines()
            out_lines = [line[:-1] for line in out_lines]
            save_name = f'{benchmark_name}_{metric_name}_results.txt'
            save_file = open(save_name, 'w')
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_lines, out_lines, lang, ckpt, model_base)
            seg_scores = [str(score)+'\n' for score in seg_scores]
            save_file.writelines(seg_scores)
            print("file is saved!")
        elif benchmark_name == 'paws':
            ref_lines = open('paws_ref.txt', 'r').readlines()
            ref_lines = [line[:-1] for line in ref_lines]
            out_lines = open('paws_better.txt', 'r').readlines()
            out_lines = [line[:-1] for line in out_lines]
            # save the better scores
            save_name = f'{benchmark_name}_{metric_name}_better_results.txt'
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_lines, out_lines, lang, ckpt, model_base)
            out_lines = open('paws_worse.txt', 'r').readlines()
            out_lines = [line[:-1] for line in out_lines]
            # save the worse scores
            save_name = f'{benchmark_name}_{metric_name}_worse_results.txt'
            save_file = open(save_name, 'w')
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_lines, out_lines, lang, ckpt, model_base)
            seg_scores = [str(score)+'\n' for score in seg_scores]
            save_file.writelines(seg_scores)
            print("files are saved!")
        elif benchmark_name == 'caption':
            ref_out_lines = open(caption_file, 'r').readlines()
            file_index = caption_file.split('.')[0][-1]
            cap_prefix = caption_file.split('.')[0].split('_')[0]
            index_ls = [ele.split('\t')[0] for ele in ref_out_lines]
            ref_ls = [ele.split('\t')[1] for ele in ref_out_lines]
            out_ls = [ele.split('\t')[2][:-1] for ele in ref_out_lines]
            save_name = f'{benchmark_name}_{cap_prefix}_{metric_name}_results_{model_base}_{file_index}.txt'
            save_file = open(save_name, 'w')
            seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_ls, out_ls, lang, ckpt, model_base)
            seg_scores = [str(score)+'\n' for score in seg_scores]
            save_file.writelines(seg_scores)
        elif benchmark_name == 'webnlg2020':
            sys_score_output_dict = json.load(open('webnlg2020.json'))
            sys_seg_scores_dict = {}
            for sys in sys_score_output_dict:
                out_ls = list(sys_score_output_dict[sys]['Output'].values())
                ref_ls = list(sys_score_output_dict[sys]['Ref'].values())
                seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, ref_ls, out_ls, lang, ckpt, model_base)
                sys_seg_scores_dict[sys] = seg_scores
            with open(f'{benchmark_name}_{metric_name}_results.json', 'w') as f:
                json.dump(sys_seg_scores_dict, f)
            print("Metric outputs are saved!")
        elif benchmark_name == 'webnlg':
            df = pd.read_csv('./webnlg-human-evaluation/all_data_final_averaged.csv')
            team_segid_scores = {}
            for i in range(3):
                ref1_ls = open(f'./webnlg-human-evaluation/gold-sample-reference{i}.lex', 'r').readlines()
                for team in set(df['team']):
                    cur_data = df.loc[df['team'] == team]
                    outs = []
                    refs = []
                    srcs = open('./webnlg-human-evaluation/MRs.txt', 'r').readlines()
                    assert(len(srcs)==223)

                    assert(len(ref1_ls)==223)
                    index_ls = []
                    for index, sen in enumerate(ref1_ls):
                        if sen != '\n':
                            index_ls += [index]
                    srcs = [src[:-1] for src in srcs[:-1]]+[srcs[-1]]
                    src_ref_mapping = {src.strip(): ref for src, ref in zip(srcs, ref1_ls)}

                    print(len(cur_data['mr']))
                    for mr_src, out_ele in zip(cur_data['mr'], cur_data['text']):
                        if src_ref_mapping[mr_src.strip()] != '\n':
                            refs.append(src_ref_mapping[mr_src])
                            if pd.isna(out_ele):
                                outs.append('')
                            else:
                                outs.append(out_ele)

                    seg_scores = test_save_metric_benchmark(metric_name, benchmark_name, refs, outs, lang, ckpt, model_base)

                    print("after seg gen")

                    if team not in team_segid_scores:
                        team_segid_scores[team] = {}
                    for index, ele_score in zip(index_ls, seg_scores):
                        if index not in team_segid_scores[team]:
                            team_segid_scores[team][index] = []
                        team_segid_scores[team][index]+=[ele_score]

            final_dict = {}
            # print(team_segid_scores)
            for team, id_score_dict in team_segid_scores.items():
                if team not in final_dict:
                    final_dict[team] = {}
                for id, score_ls in id_score_dict.items():
                    final_dict[team][id] = max(score_ls)

            save_name = f'{benchmark_name}_{metric_name}_results_{model_base}.json'
            with open(save_name, 'w') as f:
                json.dump(final_dict, f)

            print(f"{save_name} is saved!")
        elif benchmark_name == 'x-en-wmt20-da':
            import numpy as np
            data = pd.read_csv('2020-daRR.csv').dropna()
            dict_data = pd.read_csv('2020-da.csv')
            print(set(data['lp']))

            cur_lang_data = data.loc[data['lp'] == lp]
            cur_dict_data = dict_data.loc[dict_data['lp'] == lp]

            new_dict_data={}
            for mt, mt_score in zip(cur_dict_data['mt'], cur_dict_data['score']):
                if mt not in new_dict_data:
                    new_dict_data[mt]=[]
                new_dict_data[mt]+=[float(mt_score)]

            cur_lang_data_1 = cur_lang_data[cur_lang_data['better.model']=='OPPO.1360']
            cur_lang_data_1 = cur_lang_data_1[cur_lang_data_1['worse.model']=='Tohoku-AIP-NTT.1442']

            cur_lang_data_2 = cur_lang_data[cur_lang_data['better.model']=='Tohoku-AIP-NTT.1442']
            cur_lang_data_2 = cur_lang_data_2[cur_lang_data_2['worse.model']=='OPPO.1360']

            cur_lang_data = pd.concat([cur_lang_data_1, cur_lang_data_2], ignore_index=True, axis=0)

            print(cur_lang_data)

            new_dict_data = {key: sum(val_ls)/len(val_ls) for key, val_ls in new_dict_data.items()}
            tot_better_ls, tot_worse_ls, tot_ref_ls, tot_src_ls = [], [], [], []
            print(len(cur_lang_data['lp']))
            for better_ele, worse_ele, ref_ele, src_ele in zip(cur_lang_data['better'], cur_lang_data['worse'], \
                cur_lang_data['ref'], cur_lang_data['src']):
                if new_dict_data[better_ele] > new_dict_data[worse_ele]+25.0:
                    tot_better_ls+=[better_ele]
                    tot_worse_ls+=[worse_ele]
                    tot_ref_ls+=[ref_ele]
                    tot_src_ls+=[src_ele]

            print("length of evaluation: ", len(tot_better_ls))
            refs, better_outs, worse_outs = tot_ref_ls, tot_better_ls, tot_worse_ls  # cur_lang_data['ref'], cur_lang_data['better'], cur_lang_data['worse']

            better_segs = test_save_metric_benchmark(metric_name, benchmark_name, refs, better_outs, ckpt, tot_src_ls, model_base)
            save_better_segs = [str(score)+'\n' for score in better_segs]
            save_better = open(f'{benchmark_name}_{metric_name}_{model_base}_better.txt', 'w')
            save_better.writelines(save_better_segs)

            worse_segs = test_save_metric_benchmark(metric_name, benchmark_name, refs, worse_outs, ckpt, tot_src_ls, model_base)
            save_worse_segs = [str(score)+'\n' for score in worse_segs]
            save_worse = open(f'{benchmark_name}_{metric_name}_{model_base}_worse.txt', 'w')
            save_worse.writelines(save_worse_segs)

            # calculate using Kendall Tau-like correlations
            diff = np.array(better_segs)-np.array(worse_segs)
            concordant, total = np.sum(diff>0), len(better_segs)
            discordant = total-concordant
            print(concordant)
            print(total)
            print("Kendall Tau-like: ", (concordant-discordant)/total)
        else:
            print("benchmark is not supported!")

if __name__ == "__main__":
    main()



import torch
import click
import nltk
from syn_data.biimt import exp_config, preprocess_data
from transformers import (
    AutoTokenizer
)
import random

""" CUDA_VISIBLE_DEVICES=0 python3 syn_data/epoch_bleu_eval.py -model_checkpoint mt_infilling_weights_first_trial/mt_infilling_weights_zh-en_2M_1660468959/epoch0.ckpt
-src_dev_file zh_en_mask_src_0.7_2.txt -ref_dev_file zh_en_mask_tar_0.7_2.txt """

@click.command()
@click.option('-batch_size', type=int, default=64)
@click.option('-model_checkpoint', type=str)
@click.option('-tok_checkpoint', type=str, default="mbart-large-50-many-to-one-mmt-tok")
@click.option('-src_dev_file', type=str, help="addr to the src dev file", default='zh_en_mask_src_0.7_2.txt')
@click.option('-ref_dev_file', type=str, help="addr to the ref dev file", default='zh_en_mask_tar_0.7_2.txt')
def main(batch_size, model_checkpoint, tok_checkpoint, src_dev_file, ref_dev_file):
    torch.cuda.empty_cache()
    device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    tokenizer = AutoTokenizer.from_pretrained(tok_checkpoint)
    num_added_toks = tokenizer.add_special_tokens({'additional_special_tokens': ['<sep>']})
    model = torch.load(model_checkpoint).to(device)
    model.eval()
    # load in dev data
    mask_inp_dev, mask_ref_dev = open(src_dev_file, 'r').readlines(), open(ref_dev_file, 'r').readlines()
    mask_inp_dev, mask_ref_dev = [sen[:-1] for sen in mask_inp_dev], [sen[:-1] for sen in mask_ref_dev]
    dev_dataloader = preprocess_data(mask_inp_dev, mask_ref_dev, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False)

    gen_dev = []
    gen_file = open(f'{model_checkpoint}.outputs', 'w')
    with torch.no_grad():
        for dev_batch in dev_dataloader:
            # print(dev_batch['input_ids'].size())
            translated_tokens = model.generate(dev_batch['input_ids'].to(device))
            batch_texts = tokenizer.batch_decode(translated_tokens, skip_special_tokens=True)
            gen_dev.extend(batch_texts)

    save_gen_dev = [sen+'\n' for sen in gen_dev]
    gen_file.writelines(save_gen_dev)
    print("Files are generated which contains generated outputs!")

    total_bleu = 0
    for gen_sen, ref_sen in zip(gen_dev, mask_ref_dev):
        total_bleu+=nltk.translate.bleu_score.sentence_bleu([ref_sen], gen_sen)
    sen_bleu = total_bleu/len(mask_ref_dev)

    print(f"BLEU Score: {sen_bleu}")

if __name__ == "__main__":
    main()



import subprocess

def download_from_drive(FILE_ID='1XBfjvNbm5tpxdD62_gVpKQhwwrqXyewt', FILENAME='sescore2_en_supervised_3B.ckpt'):
    command = f'''wget --load-cookies /tmp/cookies.txt "https://docs.google.com/uc?export=download&confirm=$(wget --quiet --save-cookies /tmp/cookies.txt --keep-session-cookies --no-check-certificate 'https://docs.google.com/uc?export=download&id={FILE_ID}' -O- | sed -rn 's/.*confirm=([0-9A-Za-z_]+).*/\\1\\n/p')&id={FILE_ID}" -O {FILENAME} && rm -rf /tmp/cookies.txt'''
    subprocess.run(command, shell=True, check=True)



import click
import numpy as np
import scipy

def s2t_en_ja_benchmark(seg_scores, gt_lines):
    from mt_metrics_eval import data
    evs = data.EvalSet('wmt21.news', 'zh-en')
    mt_score_dict, gt_score_dict = {}, {}

    mt_score_dict['sys1'] = seg_scores[:78]
    mt_score_dict['sys2'] = seg_scores[78:78*2]
    mt_score_dict['sys3'] = seg_scores[78*2:78*3]
    mt_score_dict['sys4'] = seg_scores[78*3:]

    gt_score_dict['sys1'] = gt_lines[:78]
    gt_score_dict['sys2'] = gt_lines[78:78*2]
    gt_score_dict['sys3'] = gt_lines[78*2:78*3]
    gt_score_dict['sys4'] = gt_lines[78*3:]

    mqm_bp = evs.Correlation(gt_score_dict, mt_score_dict, mt_score_dict.keys())
    return mqm_bp.Kendall()[0]

def WilliamsTest(r12, r13, r23, n, one_sided=True):
    """Return Williams test p-value for given Pearson correlations."""
    k = 1 - r12**2 - r13**2 - r23**2 + 2 * r12 * r23 * r13
    rbar = ((r12 + r13) / 2)
    tnum = (r12 - r13) * np.sqrt((n - 1) * (1 + r23))
    tden = np.sqrt(2 * (n - 1) / (n - 3) * k + rbar**2 * (1 - r23)**3)
    p = scipy.stats.t.sf(np.abs(tnum / tden), n - 3)
    return p if one_sided else 2 * p

@click.command()
@click.option('-file_first')
@click.option('-file_sec')
def main(file_first, file_sec):
    gt_lines = open('mqm_en_ja/mqm_en_ja_scores.txt', 'r').readlines()
    gt_lines = [-float(line[:-1]) for line in gt_lines]
    seg_scores_1 = open(file_first, 'r').readlines()
    seg_scores_1 = [float(score[:-1]) for score in seg_scores_1]
    r1 = s2t_en_ja_benchmark(seg_scores_1, gt_lines)

    seg_scores_2 = open(file_sec, 'r').readlines()
    seg_scores_2 = [float(score[:-1]) for score in seg_scores_2]
    r2 = s2t_en_ja_benchmark(seg_scores_2, gt_lines)

    r12 = s2t_en_ja_benchmark(seg_scores_2, seg_scores_1)
    n = len(gt_lines)

    print(WilliamsTest(r1, r2, r12, n, one_sided=True))

if __name__ == "__main__":
    main()



import torch
import click
import torch.nn as nn
from tqdm.auto import tqdm
from transformers import AutoModelForSeq2SeqLM, AutoTokenizer
import math
import os

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

# nohup python3 mbart.py -src_lang zh_CN -tar_lang en_XX -src_file news_data/news_comp_zh_en.zh -batch_size 128 -gpu_index 1

@click.command()
@click.option('-src_lang', type=str)
@click.option('-tar_lang', type=str)
@click.option('-src_file', type=str)
@click.option('-batch_size', type=int)
@click.option('-gpu_index', type=int)
def main(src_lang, tar_lang, src_file, batch_size, gpu_index):
    print("Total number of GPUs: ", torch.cuda.device_count())
    print(f"Using GPU: {gpu_index}")

    # check if stored dir is there and make one if not
    if not os.path.isdir('mbart_outputs'):
        os.makedirs('mbart_outputs')

    total_num_gpus = torch.cuda.device_count()
    device = torch.device(f"cuda:{gpu_index}") if torch.cuda.is_available() else torch.device("cpu")

    if tar_lang == 'en_XX':
        checkpoint = "facebook/mbart-large-50-many-to-one-mmt"
    elif tar_lang == 'de_DE':
        checkpoint = "facebook/mbart-large-50-one-to-many-mmt"
    else:
        print("We currently only support en_XX and de_DE for target language")

    # load in MBart model
    model = AutoModelForSeq2SeqLM.from_pretrained(checkpoint)
    model = model.to(device)
    model.eval()
    # use Mbart tokenizer
    tokenizer = AutoTokenizer.from_pretrained(checkpoint)
    tokenizer.src_lang = src_lang
    src_ls = open(src_file, 'r').readlines()
    # determine the start line and end line index for current gpu
    cur_gpu_size = math.ceil(len(src_ls)/total_num_gpus)
    line_index_ls = list(range(0, len(src_ls), cur_gpu_size))
    start_index, end_index = line_index_ls[gpu_index], min(line_index_ls[gpu_index]+cur_gpu_size, len(src_ls))
    # rewrite the src_ls into current gpu scope
    src_ls = src_ls[start_index:end_index]
    # save four levels of scores
    saveFile = open(f'mbart_outputs/{src_lang}_{tar_lang}_{start_index}_{end_index}.txt', 'w')

    for batch_text in batchify(src_ls, batch_size=batch_size):
        with torch.no_grad():
            batch = tokenizer(batch_text, return_tensors="pt", max_length=256, truncation=True, padding=True)['input_ids'].to(device)
            translated = model.generate(batch, forced_bos_token_id=tokenizer.lang_code_to_id[tar_lang])
            translation = tokenizer.batch_decode(translated, skip_special_tokens=True)
            for translation_ele in ziptranslation:
                saveFile.write(translation_ele+'\n')

    print(f"Outputs are saved in mbart_outputs/{src_lang}_{tar_lang}_{start_index}_{end_index}_{num_beam}.txt !")

if __name__ == "__main__":
    main()



from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer
import random
import click
from dbdict import DbDict
import nltk
import click
nltk.download('punkt')
# 1) Sample five non-overlapping locations in a sentence to construct synthesized dataset
# 2) For each location, choose operations from Repeat, Delete, Swap, Insert and Replace
#    (For Replace operaiton, we use both in batch and also dictionary replacement)
# 3) Use two losses: multi-level margin losses and compactness losses

"""code to run: python3 syn_data/mt_mask_data.py -inp_addr train_400k.en -num_turns 5"""

# construct synthesized data from delete, swap, repeat, insert and replace operations
def ssl_stratified_data_schedule(words_ls, num_turns, max_span):
    indices_ls = list(range(len(words_ls)))
    # dynamically store the indices that can be modified
    temp_ls = indices_ls.copy()
    # dynamically store the amount of operations that can be added on each position
    edit_queue = [len(words_ls)-ele for ele in indices_ls]
    # 0: delete, 1: insert_mt, 3: replace_mt
    operation_type = [0, 1, 2]
    weights=[1,1,1]

    # randomly select operation at each turn
    ops_ls = []
    for i in range(num_turns):
        cur_op = random.choices(operation_type, weights=weights, k=1)[0]
        # the last operation is swap
        ops_ls.append(cur_op)
    # randomly select edit position but non-overlapping to the last operation
    edit_locs = []
    for op_index, op in enumerate(ops_ls):
        # this is for operation that needs two spans (like swap) in one sentence
        if len(temp_ls) > 0:
            rand_index = random.choice(temp_ls)
            for i in range(rand_index-1, -1, -1):
                if edit_queue[i] == 0:
                    break
                edit_queue[i] -= edit_queue[rand_index]
            # for insert operation only because insert has no error span from original sentence
            if op == 1:
                # insert position should be fixed
                temp_ls = list(set(temp_ls) - set([indices_ls[rand_index]]))
                edit_queue[rand_index] = 0
                edit_locs.append([rand_index])
            else:
                if min(max_span, edit_queue[rand_index]) > 1:
                    span_len = random.randrange(1, min(max_span, edit_queue[rand_index]))
                else:
                    span_len = 1
                temp_ls = list(set(temp_ls) - set(indices_ls[rand_index:rand_index+span_len]))
                edit_queue[rand_index:rand_index+span_len] = [0]*span_len
                edit_locs.append([rand_index, rand_index+span_len])
        else:
            # number of turns is not supported by current sentence
            ret_ops = ops_ls[:op_index]
            return ret_ops, edit_locs
    return ops_ls, edit_locs

@click.command()
@click.option('-inp_addr', type=str)
@click.option('-num_turns', type=int)
@click.option('-language', type=str, help="english or german")
def main(inp_addr, num_turns, language):
    max_span = 7
    lines = open(inp_addr, 'r').readlines()
    lines = [line[:-1] for line in lines]
    save_files_mt_ls, save_files_del_ls = [], []
    num_turn_choices = list(range(1, num_turns+1))
    # generate save file
    for i in range(1, num_turns+1, 1):
        save_files_mt_ls.append(open(f'mt_sen_op_loc_{i}.txt', 'w'))
    for i in range(1, num_turns+1, 1):
        save_files_del_ls.append(open(f'del_sen_op_loc_{i}.txt', 'w'))

    for sen_index, line in enumerate(lines):
        cur_num_turns = random.choice(num_turn_choices)
        words_ls = word_tokenize(line, language)
        ops_ls, edit_locs = ssl_stratified_data_schedule(words_ls, cur_num_turns, max_span)
        # save as (sen_index, op, edit_locs)
        for file_index, (op, loc) in enumerate(zip(ops_ls, edit_locs)):
            if len(loc) == 1:
                cur_loc = f'{loc[0]}'
            elif len(loc) == 2:
                cur_loc = f'{loc[0]}_{loc[1]}'
            else:
                print("Errors in sentence locations")
                exit(1)

            if op == 0:
                save_files_del_ls[file_index].write(str(sen_index)+'\t'+str(op)+'\t'+cur_loc+'\n')
            else:
                save_files_mt_ls[file_index].write(str(sen_index)+'\t'+str(op)+'\t'+cur_loc+'\n')

if __name__ == "__main__":
    main()



import click
import torch
import torch.nn as nn
import wandb
from datasets import load_dataset
from torch.utils.data import DataLoader
from datasets import Dataset
from transformers import (
    AutoConfig,
    AutoTokenizer,
    AutoModelForSeq2SeqLM,
    DataCollatorWithPadding,
    get_linear_schedule_with_warmup,
    AdamW
)
import math
from torch.nn.parallel import DistributedDataParallel as DDP
import torch.distributed as dist
import os
from scipy.stats import poisson
import nltk
from nltk.tokenize import word_tokenize
from random import randrange
from nltk.tokenize.treebank import TreebankWordDetokenizer
from tqdm.auto import tqdm
import random

nltk.download('punkt')

class exp_config():
    max_length = 256
    hidden_size = 1024
    temp = 0.05
    lr = 3e-05

def construct_uniform_mask_data(src_data, ref_data, mask_token, sep_token, mask_ratio, aug_factor):
    src_data, ref_data = [data[:-1] for data in src_data], [data[:-1] for data in ref_data]
    mask_inp_data_ls, mask_ref_data_ls = [], []
    with tqdm(total=len(src_data)*aug_factor) as pbar:
        for src_sen, ref_sen in zip(src_data, ref_data):
            for i in range(aug_factor):
                mask_inp_data, mask_tar_data = [], []
                ref_words_ls = word_tokenize(ref_sen)
                mask_len = int(len(ref_words_ls) * mask_ratio)
                start_index = randrange(0, len(ref_words_ls)-mask_len+1)
                end_index = start_index + mask_len
                mask_inp_data+=ref_words_ls[:start_index] + [mask_token] + ref_words_ls[end_index:]
                mask_tar_data+=ref_words_ls[start_index:end_index]
                mask_inp_data_ls.append(src_sen +f' {sep_token} '+ TreebankWordDetokenizer().detokenize(mask_inp_data))
                mask_ref_data_ls.append(TreebankWordDetokenizer().detokenize(mask_tar_data))
                pbar.update(1)
    return mask_inp_data_ls, mask_ref_data_ls

# in data preprocess, we need to randomnized the mask function in every epoch
def preprocess_data(src_data, ref_data, tokenizer, max_length, batch_size, shuffle=True, sampler=True):
    ds = Dataset.from_dict({"src": src_data, 'tar': ref_data})
    # generate input data
    def preprocess_function(examples):
        model_inputs = {}
        srcs = tokenizer(examples['src'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['input_ids'], model_inputs['src_attn_masks'] = srcs['input_ids'], srcs['attention_mask']
        targets = tokenizer(examples['tar'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['labels'] = targets["input_ids"]
        return model_inputs

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors='pt'
    )

    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

def store_nli_loss(model, dev_dataloader, train_batch):
    model.eval()
    with torch.no_grad():
        train_loss = model(train_batch['input_ids'], attention_mask=train_batch['src_attn_masks'], labels=train_batch['labels']).loss
        dev_loss = 0
        # store all dev loss
        for dev_batch in dev_dataloader:
            dev_loss += model(dev_batch['input_ids'], attention_mask=dev_batch['src_attn_masks'], labels=dev_batch['labels']).loss
        dev_loss = dev_loss/len(dev_dataloader)
        # log both training and dev losses in wandb
        wandb.log({
                   "training loss": train_loss.item(),
                   "dev loss": dev_loss.item()
                  })
    model.train()
    return dev_loss

@click.command()
@click.option('-aug_factor', type=int)
@click.option('-tar_lang')
@click.option('-mask_ratio', type=float, default=0.7)
@click.option('-gradient_accumulation_steps', default=8, type=int)
@click.option('-src_train_file', type=str, help="addr to the src train file", default='mt_infilling_data/zh-en/2M/zh_train_2M.txt')
@click.option('-ref_train_file', type=str, help="addr to the ref train file", default='mt_infilling_data/zh-en/2M/en_train_2M.txt')
@click.option('-src_dev_file', type=str, help="addr to the src dev file", default='mt_infilling_data/zh-en/2M/zh_dev_2M.txt')
@click.option('-ref_dev_file', type=str, help="addr to the ref dev file", default='mt_infilling_data/zh-en/2M/en_dev_2M.txt')
@click.option('-batch_size', type=int, help="batch size for contrastive learning", default=32)
@click.option('-num_epoch', type=int, help="Number of epoches to train", default=5)
@click.option('-eval_step', type=int, help="Number of steps to evaluate", default=500)
@click.option('-num_warmup_steps', type=int, help="Number of steps to warm up", default=0)
@click.option('-save_dir_name', type=str, help="the dir name of weights being saved", default=None)
def main(aug_factor, tar_lang, mask_ratio, src_train_file, ref_train_file, src_dev_file, ref_dev_file, batch_size, \
num_epoch, eval_step, num_warmup_steps, gradient_accumulation_steps, save_dir_name):
    random.seed(10)
    # initalize the process
    dist.init_process_group(backend="nccl")
    rank = int(os.environ['LOCAL_RANK'])
    # only main process initalize wandb
    if rank == 0:
        # initalize the project parameters into Wandb, store experiment specific parameters
        wandb.init(project="ContraScore", config=
        {
            "epoch": num_epoch,
            "eval_step": eval_step,
            "batch size": batch_size * gradient_accumulation_steps,
            "num_warmup_steps": num_warmup_steps,
            "mask ratio": mask_ratio,
            "aug factor": aug_factor,
            "seed": 10
        })
    exp_config.device_id = rank % torch.cuda.device_count()

    if tar_lang == 'en_XX':
        model_checkpoint = "mbart-large-50-many-to-one-mmt-model"
        tok_checkpoint = "mbart-large-50-many-to-one-mmt-tok"
    elif tar_lang == 'de_DE':
        model_checkpoint = "mbart-large-50-one-to-many-mmt-model"
        tok_checkpoint = "mbart-large-50-one-to-many-mmt-tok"
    else:
        print("We currently only support en_XX and de_DE for target language")
        exit(1)

    # set cuda device with rank and clear ram cache to ensure balanced ram allocations
    torch.cuda.set_device(rank)
    torch.cuda.empty_cache()
    tokenizer = AutoTokenizer.from_pretrained(tok_checkpoint)
    num_added_toks = tokenizer.add_special_tokens({'additional_special_tokens': ['<sep>']})
    model = AutoModelForSeq2SeqLM.from_pretrained(model_checkpoint)
    model.resize_token_embeddings(len(tokenizer))
    model = model.to(exp_config.device_id)

    print("load in models")

    # parallelize the pipeline into multiple gpus
    optimizer = AdamW(model.parameters(), lr=exp_config.lr)
    # load in src,ref and mt data if available
    src_train_data, src_dev_data = open(src_train_file, 'r').readlines(), open(src_dev_file, 'r').readlines()
    ref_train_data, ref_dev_data = open(ref_train_file, 'r').readlines(), open(ref_dev_file, 'r').readlines()
    mask_inp_train, mask_ref_train = construct_uniform_mask_data(src_train_data, ref_train_data, tokenizer.mask_token, '<sep>', mask_ratio, aug_factor)
    mask_inp_dev, mask_ref_dev = construct_uniform_mask_data(src_dev_data, ref_dev_data, tokenizer.mask_token, '<sep>', mask_ratio, aug_factor)

    print('construct masked data!')

    train_dataloader = preprocess_data(mask_inp_train, mask_ref_train, tokenizer, exp_config.max_length, batch_size)
    dev_dataloader = preprocess_data(mask_inp_dev, mask_ref_dev, tokenizer, exp_config.max_length, batch_size)

    model = DDP(model, device_ids=[exp_config.device_id])
    model.train()

    max_train_steps = math.ceil(len(train_dataloader) / gradient_accumulation_steps) * num_epoch
    lr_scheduler = get_linear_schedule_with_warmup(
        optimizer=optimizer,
        num_warmup_steps=num_warmup_steps,
        num_training_steps=max_train_steps,
    )

    if not os.path.isdir(f'{save_dir_name}') and rank == 0:
        os.makedirs(f'{save_dir_name}')

    with torch.autograd.set_detect_anomaly(True):
        for epoch in range(num_epoch):
            # store best checkpoint and end-of-epoch checkpoint at each epoch folder
            if not os.path.isdir(f'{save_dir_name}/epoch{epoch}') and rank == 0:
                os.makedirs(f'{save_dir_name}/epoch{epoch}')
                dev_loss_file = open(f"{save_dir_name}/epoch{epoch}/epoch{epoch}_dev.loss", 'w')
            step_best_dev_loss = float("inf")
            torch.cuda.empty_cache() # empty cache in gpus
            train_dataloader.sampler.set_epoch(epoch) # set the sampler at each epoch
            for step, train_batch in enumerate(train_dataloader):
                # evaluate at every eval_step (includes the beginning loss)
                if (step % (eval_step * gradient_accumulation_steps) == 0) and rank == 0:
                    print(f"In the eval step: {step}")
                    # store all the losses in wandb
                    cur_dev_loss = store_nli_loss(model, dev_dataloader, train_batch)
                    if cur_dev_loss < step_best_dev_loss:
                        step_best_dev_loss = cur_dev_loss
                        torch.save(model.module, f'{save_dir_name}/epoch{epoch}/epoch{epoch}_best.ckpt')
                        dev_loss_file.write(f'Step {step}, {cur_dev_loss}\n')

                train_loss = model(train_batch['input_ids'], attention_mask=train_batch['src_attn_masks'], labels=train_batch['labels']).loss
                # accumulate losses at weights, each is normalized by accumulation steps
                train_loss = train_loss / gradient_accumulation_steps
                train_loss.backward()

                if step % gradient_accumulation_steps == 0 or step == len(train_dataloader) - 1:
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad() # clear the grads

                # save at end of epoch and at main processls
                if step == len(train_dataloader) - 1 and rank == 0:
                    cur_dev_loss = store_nli_loss(model, dev_dataloader, train_batch)
                    torch.save(model.module, f'{save_dir_name}/epoch{epoch}/epoch{epoch}_end.ckpt')
                    dev_loss_file.write(f'End of Epoch: {cur_dev_loss}')
                    dev_loss_file.close()
                    print(f"Saved entire model at current epoch {epoch}!")

if __name__ == "__main__":
    main()
